package com.sslexplorer.agent.client.util;

import static org.junit.Assert.*;

import org.junit.Test;

public class UtilsTest {

	@Test
	public void testIsSupportedJREGreatherThan() {
		System.setProperty("java.version", "1.4");
		assertTrue("1.3 supported by 1.4", Utils.isSupportedJRE("+1.3"));
		assertTrue("1.4 supported by 1.4", Utils.isSupportedJRE("+1.4"));
		assertFalse("1.4.5 not supported by 1.4", Utils.isSupportedJRE("+1.4.5"));
		assertFalse("1.5 not supported by 1.4", Utils.isSupportedJRE("+1.5"));
		assertFalse("1.5.0 not supported by 1.4", Utils.isSupportedJRE("+1.5.0"));

		System.setProperty("java.version", "1.1.1");
		assertTrue("1.0 is supported by 1.1.1", Utils.isSupportedJRE("+1.0"));
		assertTrue("1.1.1 supported by 1.1.1", Utils.isSupportedJRE("1.1.1"));
		assertTrue("1.1.1 supported by 1.1.1", Utils.isSupportedJRE("+1.1.1"));
		assertFalse("1.1.2 not supported by 1.1.1", Utils.isSupportedJRE("+1.1.2"));
		assertFalse("1.4.5 not supported by 1.1.1", Utils.isSupportedJRE("+1.4.5"));
	}

	@Test
	public void testIsSupportedJRELessThan() {
		System.setProperty("java.version", "1.4");
		assertFalse("-1.3 not supported by 1.4", Utils.isSupportedJRE("-1.3"));
		assertTrue("-1.4 supported by 1.4", Utils.isSupportedJRE("-1.4"));
		assertTrue("-1.5 supported by 1.4", Utils.isSupportedJRE("-1.5"));

		System.setProperty("java.version", "1.1.4");
		assertTrue("-1.3 not supported by 1.1.4", Utils.isSupportedJRE("-1.3"));
		assertFalse("+1.3 not supported by 1.1.4", Utils.isSupportedJRE("+1.3"));
	}

	@Test
	public void testIsSupportedOSVersionLessThan() {
		System.setProperty("os.version", "6.0");
		assertFalse("-5.0 not supported by 6.0", Utils.isSupportedOSVersion("-5.0"));
		assertFalse("-5.9 not supported by 6.0", Utils.isSupportedOSVersion("-5.9"));
		assertTrue("6.0 supported by 6.0", Utils.isSupportedOSVersion("-6.0"));
		assertTrue("6.1 supported by 6.0", Utils.isSupportedOSVersion("-6.1"));
	}
}
