/*
 *  SSL-Explorer
 *
 *  Copyright (C) 2003-2006 3SP LTD. All Rights Reserved
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2 of
 *  the License, or (at your option) any later version.
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public
 *  License along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
			
package com.sslexplorer.activedirectory;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.sslexplorer.security.Role;

final class RoleMap implements Serializable {
    private static final long serialVersionUID = -7675489973639150879L;
    private static final Log log = LogFactory.getLog(RoleMap.class);
    private final Map<String, ActiveDirectoryGroup> dnToRoleMap = new TreeMap<String, ActiveDirectoryGroup>();
    private final Map<Long, ActiveDirectoryGroup> groupsByRID = new TreeMap<Long, ActiveDirectoryGroup>();
    private final Map<String, List<String>> parentGroupsByDN = new TreeMap<String, List<String>>();

    Role getByDN(String dn) {
        return dnToRoleMap.get(dn.toLowerCase());
    }

    boolean containsDN(String dn) {
        return dnToRoleMap.containsKey(dn.toLowerCase());
    }

    boolean isEmpty() {
        return dnToRoleMap.isEmpty();
    }

    int size() {
        return dnToRoleMap.size();
    }

    Collection<ActiveDirectoryGroup> values() {
        return dnToRoleMap.values();
    }

    void put(ActiveDirectoryGroup role, String dn, List<String> parents) {
        dnToRoleMap.put(dn.toLowerCase(), role);
        if (role.getRID() != null) { // Only NT Authority groups will be used
            groupsByRID.put(role.getRID(), role);
        }
        parentGroupsByDN.put(role.getOriginalDn(), parents);
    }

    ActiveDirectoryGroup getByRID(Long rid) {
        return groupsByRID.get(rid);
    }

    void buildHierarchy() {
        for (Map.Entry<String, List<String>> entry : parentGroupsByDN.entrySet()) {
            String roleDn = entry.getKey().toLowerCase();
            ActiveDirectoryGroup role = dnToRoleMap.get(roleDn);
            if (role != null) {
                List<String> parents = entry.getValue();

                List<ActiveDirectoryGroup> groups = new ArrayList<ActiveDirectoryGroup>();
                for (String parentDn : parents) {
                    parentDn = parentDn.toLowerCase();

                    if (dnToRoleMap.containsKey(parentDn)) {
                        ActiveDirectoryGroup foundRole = dnToRoleMap.get(parentDn);
                        if (foundRole != null) {
                            groups.add(foundRole);
                        } else if (log.isDebugEnabled())
                            log.debug("Found NULL role in parent list");
                    }
                }
                role.setParents(groups.toArray(new ActiveDirectoryGroup[groups.size()]));
            }
        }
    }
}