/*
 *  SSL-Explorer
 *
 *  Copyright (C) 2003-2006 3SP LTD. All Rights Reserved
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2 of
 *  the License, or (at your option) any later version.
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public
 *  License along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
			
package com.sslexplorer.applications;

import java.util.Properties;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.maverick.multiplex.Channel;
import com.maverick.multiplex.MultiplexedConnection;
import com.maverick.multiplex.Request;
import com.maverick.multiplex.RequestHandler;
import com.maverick.util.ByteArrayReader;
import com.maverick.util.ByteArrayWriter;
import com.sslexplorer.agent.AgentService;
import com.sslexplorer.agent.AgentTunnel;
import com.sslexplorer.agent.channels.ApplicationFileChannel;
import com.sslexplorer.core.CoreAttributeConstants;
import com.sslexplorer.core.CoreEvent;
import com.sslexplorer.core.CoreEventConstants;
import com.sslexplorer.core.CoreServlet;
import com.sslexplorer.extensions.ExtensionDescriptor;
import com.sslexplorer.extensions.ExtensionParser;
import com.sslexplorer.extensions.store.ExtensionStore;
import com.sslexplorer.policyframework.LaunchSession;
import com.sslexplorer.policyframework.LaunchSessionFactory;
import com.sslexplorer.policyframework.LaunchSessionManager;
import com.sslexplorer.policyframework.Policy;
import com.sslexplorer.policyframework.Resource;
import com.sslexplorer.policyframework.ResourceAccessEvent;

public class ApplicationService implements AgentService, RequestHandler {

	final static Log log = LogFactory.getLog(ApplicationService.class);

	public void performStartup(AgentTunnel agent) {
	}

	/**
	 * Create the request to instruct the agent to launch an application
	 * 
	 * @param httpRequest request
	 * @param launchSession launch session
	 * @return boolean indicating if the agent reported the application was
	 *         launched
	 * @throws Exception
	 */
	public Request launchApplication(LaunchSession launchSession) throws Exception {
		ApplicationShortcut shortcut = (ApplicationShortcut) launchSession.getResource();
		ByteArrayWriter msg = new ByteArrayWriter();
		msg.writeString(shortcut.getApplication());
		msg.writeInt(shortcut.getResourceId());
		msg.writeString(launchSession.getId());
		msg.writeString(ExtensionParser.processApplicationParameters(launchSession,
			new Properties(),
			shortcut.getParameters(),
			shortcut.getApplication()));
		return new Request("launchApplication@3sp.com", msg.toByteArray());
	}

	public Channel createChannel(MultiplexedConnection connection, String type) {
		if (type.equals(ApplicationFileChannel.CHANNEL_TYPE)) {
			return new ApplicationFileChannel((AgentTunnel)connection);
		} else
			return null;
	}

	public boolean processRequest(Request request, MultiplexedConnection connection) {

		AgentTunnel agent = (AgentTunnel) connection;

		if (request.getRequestName().equals("setupAndLaunchApplication@3sp.com") && request.getRequestData()!=null) {

			try {
				ByteArrayReader reader = new ByteArrayReader(request.getRequestData());
				int id = (int)reader.readInt();
				Resource resource = ApplicationsPlugin.APPLICATION_SHORTCUT_RESOURCE_TYPE.getResourceById(id);
				if (resource == null) {
					throw new Exception("No resource with ID " + id);
				}
				Policy policy = LaunchSessionManager.getLaunchRequestPolicy(null, agent.getSession(), resource);
				if (resource.sessionPasswordRequired(agent.getSession())) {
					// TODO: prompt user for credentials through agent!
					return true;
				} else {
					LaunchSession launchSession = LaunchSessionFactory.getInstance().createLaunchSession(agent.getSession(),
						resource,
						policy);
					launchSession.checkAccessRights(null, agent.getSession());

					ApplicationShortcut shortcut = (ApplicationShortcut) launchSession.getResource();
					ExtensionDescriptor descriptor = ExtensionStore.getInstance().getExtensionDescriptor(shortcut.getApplication());

					Request newRequest = launchApplication(launchSession);
					request.setRequestData(newRequest.getRequestData());

					CoreServlet.getServlet().fireCoreEvent(new ResourceAccessEvent(this,
									CoreEventConstants.APPLICATION_SHORTCUT_LAUNCHED,
									launchSession.getResource(),
									launchSession.getPolicy(),
									launchSession.getSession(),
									CoreEvent.STATE_SUCCESSFUL).addAttribute(CoreAttributeConstants.EVENT_ATTR_APPLICATION_NAME,
						descriptor.getName()).addAttribute(CoreAttributeConstants.EVENT_ATTR_APPLICATION_ID, descriptor.getId()));

				}
				return true;
			} catch (Exception e) {
				e.printStackTrace();
				return false;
			}
		} 
		return false;
	}

	public void postReply(MultiplexedConnection connection) {		
	}

	public void initializeTunnel(AgentTunnel tunnel) {
		tunnel.registerRequestHandler("setupAndLaunchApplication@3sp.com", this);
	}
}
