/*
 *  SSL-Explorer
 *
 *  Copyright (C) 2003-2006 3SP LTD. All Rights Reserved
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2 of
 *  the License, or (at your option) any later version.
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public
 *  License along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
			
package com.sslexplorer.applications;

import java.util.Map;

import com.sslexplorer.extensions.ApplicationLauncher;
import com.sslexplorer.policyframework.Resource;

/**
 * Extension of the {@link Resource} interface to describe 
 * <i>Application Shortcuts</i>. 
 * <p>
 * These resources allow the user to create shortcuts to the installed
 * <i>Application Extensions</i>
 * <p>
 * Each Application Extension will support a number of parameters. This shortcut
 * will also store the values that have been configured.
 * 
 * @author Brett Smith <a href="mailto: brett@3sp.com">&lt;brett@3sp.com&gt;</a>
 * @see com.sslexplorer.extensions.ApplicationLauncher
 */
public interface ApplicationShortcut extends Resource {
    /**
     * Get the parameters for this shortcut.
     * 
     * @return parameters
     */
    public Map<String, String> getParameters();
    
    /**
     * Get the ID of the application launcher this shortcut requires.
     * 
     * @return application
     */
    public String getApplication();

}