/*
 *  SSL-Explorer
 *
 *  Copyright (C) 2003-2006 3SP LTD. All Rights Reserved
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2 of
 *  the License, or (at your option) any later version.
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public
 *  License along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
			
package com.sslexplorer.applications.types;

import java.io.IOException;
import java.util.Iterator;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.jdom.Element;

import com.maverick.multiplex.Request;
import com.sslexplorer.agent.AgentTunnel;
import com.sslexplorer.agent.DefaultAgentManager;
import com.sslexplorer.applications.ApplicationLauncherType;
import com.sslexplorer.applications.ApplicationService;
import com.sslexplorer.applications.ApplicationShortcut;
import com.sslexplorer.extensions.ExtensionDescriptor;
import com.sslexplorer.extensions.ExtensionException;
import com.sslexplorer.policyframework.LaunchSession;
import com.sslexplorer.security.SessionInfo;

/**
 * Implementation of an
 * {@link com.sslexplorer.applications.ApplicationLauncherType} that allows
 * launching of native applications installed the client machine using a running
 * Agent.
 * 
 * @author Brett Smith <a href="mailto: brett@3sp.com">&lt;brett@3sp.com&gt;</a>
 */
public class ExecutableType implements ApplicationLauncherType {

	final static Log log = LogFactory.getLog(ExecutableType.class);

	/**
	 * Type name
	 */
	public final static String TYPE = "executable";

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.sslexplorer.extensions.ExtensionType#start(com.sslexplorer.extensions.ExtensionDescriptor,
	 *      org.jdom.Element)
	 */
	public void start(ExtensionDescriptor descriptor, Element element) throws ExtensionException {

		if (element.getName().equals(TYPE)) {
			verifyExecutable(element);
		}

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.sslexplorer.extensions.ExtensionType#verifyRequiredElements()
	 */
	public void verifyRequiredElements() throws ExtensionException {

	}

	/* (non-Javadoc)
	 * @see com.sslexplorer.applications.ApplicationLauncherType#launch(java.util.Map, com.sslexplorer.extensions.ExtensionDescriptor, com.sslexplorer.applications.ApplicationShortcut, org.apache.struts.action.ActionMapping, com.sslexplorer.policyframework.LaunchSession, java.lang.String, javax.servlet.http.HttpServletRequest)
	 */
	public ActionForward launch(Map<String, String> parameters, ExtensionDescriptor descriptor, ApplicationShortcut shortcut,
								ActionMapping mapping, LaunchSession launchSession, String returnTo, HttpServletRequest request) throws ExtensionException {
		if (log.isInfoEnabled())
			log.info("Launching client application " + shortcut.getResourceName());

		// SessionInfo session =
		// LogonControllerFactory.getInstance().getSessionInfo(request);
		if (DefaultAgentManager.getInstance().hasActiveAgent(launchSession.getSession())) {
			try {
				Request agentRequest = ((ApplicationService) DefaultAgentManager.getInstance().getService(ApplicationService.class)).launchApplication(launchSession);
				AgentTunnel agent = DefaultAgentManager.getInstance().getAgentBySession(launchSession.getSession());
				if (!agent.sendRequest(agentRequest, true, 60000)) {
					throw new ExtensionException(ExtensionException.AGENT_REFUSED_LAUNCH);
				}
			} catch (ExtensionException ee) {
				throw ee;
			} catch (Exception e) {
				throw new ExtensionException(ExtensionException.INTERNAL_ERROR, e);
			}
		} else {
			throw new ExtensionException(ExtensionException.NO_AGENT);
		}
		return null;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.sslexplorer.extensions.ExtensionType#isHidden()
	 */
	public boolean isHidden() {
		return false;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.sslexplorer.extensions.ExtensionType#getType()
	 */
	public String getType() {
		return TYPE;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.sslexplorer.extensions.ExtensionType#stop()
	 */
	public void stop() throws ExtensionException {
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.sslexplorer.applications.ApplicationLauncherType#isAgentRequired(com.sslexplorer.applications.ApplicationShortcut,
	 *      com.sslexplorer.extensions.ExtensionDescriptor)
	 */
	public boolean isAgentRequired(ApplicationShortcut shortcut, ExtensionDescriptor descriptor) {
		return true;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.sslexplorer.extensions.ExtensionType#activate()
	 */
	public void activate() throws ExtensionException {
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.sslexplorer.extensions.ExtensionType#canStop()
	 */
	public boolean canStop() throws ExtensionException {
		return true;
	}

	private void verifyExecutable(Element element) throws ExtensionException {
		for (Iterator it = element.getChildren().iterator(); it.hasNext();) {
			Element e = (Element) it.next();
			if (e.getName().equalsIgnoreCase("if")) {
				verifyExecutable(e);
			} else if (!e.getName().equalsIgnoreCase("arg") && !e.getName().equalsIgnoreCase("program") && !e.getName().equalsIgnoreCase("jvm")) {
				throw new ExtensionException(ExtensionException.FAILED_TO_PROCESS_DESCRIPTOR, "Unexpected element <" + e.getName()
					+ "> found in <executable>");
			}
		}

	}

	/* (non-Javadoc)
	 * @see com.sslexplorer.extensions.ExtensionType#descriptorCreated(org.jdom.Element)
	 */
	public void descriptorCreated(Element element, SessionInfo session) throws IOException {		
	}

	/* (non-Javadoc)
	 * @see com.sslexplorer.extensions.ExtensionType#getTypeBundle()
	 */
	public String getTypeBundle() {
		return "applications";
	}
}