/*
 *  SSL-Explorer
 *
 *  Copyright (C) 2003-2006 3SP LTD. All Rights Reserved
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2 of
 *  the License, or (at your option) any later version.
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public
 *  License along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
			
package com.sslexplorer.applicationshortctus;

import java.util.Calendar;
import java.util.Collections;
import java.util.List;

import org.junit.BeforeClass;
import org.junit.Test;

import com.sslexplorer.applications.ApplicationShortcut;
import com.sslexplorer.applications.ApplicationShortcutDatabase;
import com.sslexplorer.applications.ApplicationShortcutDatabaseFactory;
import com.sslexplorer.applications.ApplicationsPlugin;
import com.sslexplorer.applications.DefaultApplicationShortcut;
import com.sslexplorer.policyframework.ResourceType;
import com.sslexplorer.testcontainer.policyframework.AbstractTestResource;

public class ApplicationShortcutsTests extends AbstractTestResource<ApplicationShortcut> {

    @BeforeClass
    public static void oneTimeSetUp() throws Exception {
        setUp("sslexplorer-community-applications,sslexplorer-community-tunnels");
    }

    @Override
    public ApplicationShortcut getEmptyResource() throws Exception {
        Calendar calendar = Calendar.getInstance();
        return new DefaultApplicationShortcut(getDefaultRealm().getRealmID(), -1, "", "", calendar, calendar, "", Collections.EMPTY_MAP);
    }

    @Override
    public ApplicationShortcut getNormalResource() throws Exception {
        Calendar calendar = Calendar.getInstance();
        return new DefaultApplicationShortcut(getDefaultRealm().getRealmID(), -1, "RDP Name", "This is a RDP Description", calendar, calendar, "RDP", Collections.EMPTY_MAP);
    }

    @Override
    public ApplicationShortcut getNullResource() throws Exception {
        Calendar calendar = Calendar.getInstance();
        return new DefaultApplicationShortcut(getDefaultRealm().getRealmID(), -1, null, null, calendar, calendar, null, Collections.EMPTY_MAP);
    }

    @Override
    public ApplicationShortcut createResource(ApplicationShortcut resource) throws Exception {
        return getApplicationShortcutService().getShortcut(
            getApplicationShortcutService().createApplicationShortcut(resource.getApplication(), resource.getResourceName(),
                resource.getResourceDescription(), resource.getParameters(), resource.getRealmID()));
    }

    @Override
    public ApplicationShortcut updateResource(ApplicationShortcut resource) throws Exception {
        getApplicationShortcutService().updateApplicationShortcut(resource.getResourceId(), resource.getResourceName(),
            resource.getResourceDescription(), resource.getParameters());
        return getApplicationShortcutService().getShortcut(resource.getResourceId());
    }

    @Override
    public List<ApplicationShortcut> getAllResources() throws Exception {
        return getApplicationShortcutService().getShortcuts();
    }

    @Override
    public ResourceType getResourceType() throws Exception {
        return ApplicationsPlugin.APPLICATION_SHORTCUT_RESOURCE_TYPE;
    }

    @Override
    public ApplicationShortcut deleteResource(ApplicationShortcut resource) throws Exception {
        ApplicationShortcut shortcut = ApplicationShortcutDatabaseFactory.getInstance().getShortcut(resource.getResourceId());
        getApplicationShortcutService().deleteShortcut(shortcut.getResourceId());
        return shortcut;
    }

    @Override
    public ApplicationShortcut getResource(ApplicationShortcut resource) throws Exception {
        return getApplicationShortcutService().getShortcut(resource.getResourceId());
    }

    protected static ApplicationShortcutDatabase getApplicationShortcutService() throws Exception {
        return ApplicationShortcutDatabaseFactory.getInstance();
    }

    @Override
    public void createNullResource() throws Exception {
    }

    @Test
    public void parentMustHaveATest() {
    }
}