/*
 *  SSL-Explorer
 *
 *  Copyright (C) 2003-2006 3SP LTD. All Rights Reserved
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2 of
 *  the License, or (at your option) any later version.
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public
 *  License along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
			
package com.sslexplorer.tunnels.forms;

import java.util.List;

import javax.servlet.http.HttpServletRequest;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.Globals;
import org.apache.struts.action.ActionErrors;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.util.LabelValueBean;

import com.sslexplorer.boot.HostService;
import com.sslexplorer.boot.PropertyList;
import com.sslexplorer.input.MultiSelectSelectionModel;
import com.sslexplorer.policyframework.Resource;
import com.sslexplorer.policyframework.forms.AbstractFavoriteResourceForm;
import com.sslexplorer.security.User;
import com.sslexplorer.tunnels.TransportType;
import com.sslexplorer.tunnels.Tunnel;

/**
 * Implementation of {@link AbstractFavoriteResourceForm} suitable for editing
 * an <i>SSL Tunnel</i>.
 * 
 * @author James Robinson <a href="mailto: james@3sp.com">&lt;james@3sp.com&gt;</a>
 * @author Brett Smith <a href="mailto: brett@3sp.com">&lt;brett@3sp.com&gt;</a>
 */
public class TunnelForm extends AbstractFavoriteResourceForm {
    static Log log = LogFactory.getLog(TunnelForm.class);

    // Private instance variables

    private String selectedTab = "details";

    private String sourceInterface;
    private String sourcePort;
    private String destinationHost;
    private String destinationPort;
    private int tunnelType;
    private String transport;
    private boolean autoStart;

    /*
     * (non-Javadoc)
     * 
     * @see com.sslexplorer.policyframework.forms.AbstractResourceForm#validate(org.apache.struts.action.ActionMapping,
     *      javax.servlet.http.HttpServletRequest)
     */
    public ActionErrors validate(ActionMapping mapping, HttpServletRequest request) {
        ActionErrors errs = super.validate(mapping, request);
        if (isCommiting()) {
                        
            try {
                int port = Integer.valueOf(sourcePort).intValue();
                if(port < 1 || port > 65535) {
                    throw new IllegalArgumentException();
                }
            } catch (Exception e) {
                errs.add(Globals.ERROR_KEY, new ActionMessage("tunnelWizard.tunnelDetails.error.sourcePortNotInteger"));
            }
            
            try {
                int port = Integer.valueOf(destinationPort).intValue();
                if(port < 1 || port > 65535) {
                    throw new IllegalArgumentException();
                }        
                Integer.valueOf(destinationPort).intValue();
            } catch (Exception e) {
                errs.add(Globals.ERROR_KEY, new ActionMessage("tunnelWizard.tunnelDetails.error.destinationPortNotInteger"));
            }

            if (destinationHost == null || destinationHost.equals("")) {
                errs.add(Globals.ERROR_KEY, new ActionMessage("tunnelWizard.tunnelDetails.error.noDestinationHost"));
            }
            
            if (getResourceName().equalsIgnoreCase("default")
                            && (!getEditing() || (getEditing() && !getResource().getResourceName().equalsIgnoreCase("default")))) {
                errs.add(Globals.ERROR_KEY, new ActionMessage("error.createNetworkPlace.cantUseNameDefault"));
                setResourceName("");
            }
        }
        return errs;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.sslexplorer.tabs.TabModel#getTabCount()
     */
    public int getTabCount() {
        return 3;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.sslexplorer.tabs.TabModel#getTabName(int)
     */
    public String getTabName(int idx) {
        switch (idx) {
            case 0:
                return "details";
            case 1:
                return "other";
            default:
                return "policies";
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.sslexplorer.tabs.TabModel#getTabTitle(int)
     */
    public String getTabTitle(int idx) {

        // Get from resources
        return null;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.sslexplorer.policyframework.forms.AbstractFavoriteResourceForm#initialise(com.sslexplorer.security.User,
     *      com.sslexplorer.policyframework.Resource, boolean,
     *      com.sslexplorer.input.MultiSelectSelectionModel,
     *      com.sslexplorer.boot.PropertyList, com.sslexplorer.security.User,
     *      boolean)
     */
    public void initialise(User user, Resource resource, boolean editing, MultiSelectSelectionModel policyModel,
                    PropertyList selectedPolicies, User owner, boolean assignOnly) throws Exception {
        super.initialise(user, resource, editing, policyModel, selectedPolicies, owner, assignOnly);
        Tunnel tunnel = (Tunnel) resource;
        sourcePort = String.valueOf(tunnel.getSourcePort());
        destinationHost = tunnel.getDestination().getHost();
        destinationPort = String.valueOf(tunnel.getDestination().getPort());
        tunnelType = tunnel.getType();
        transport = tunnel.getTransport();
        autoStart = tunnel.isAutoStart();
        sourceInterface = tunnel.getSourceInterface();
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.sslexplorer.tabs.TabModel#getSelectedTab()
     */
    public String getSelectedTab() {
        return selectedTab;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.sslexplorer.tabs.TabModel#setSelectedTab(java.lang.String)
     */
    public void setSelectedTab(String selectedTab) {
        this.selectedTab = selectedTab;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.sslexplorer.policyframework.forms.AbstractResourceForm#applyToResource()
     */
    public void applyToResource() throws Exception {
        Tunnel tunnel = (Tunnel) getResource();
        tunnel.setType(getTunnelType());
        tunnel.setAutoStart(isAutoStart());
        tunnel.setTransport(getTransport());
        tunnel.setSourcePort(Integer.parseInt(getSourcePort()));
        tunnel.setDestination(new HostService(getDestinationHost(), Integer.parseInt(getDestinationPort())));
        tunnel.setSourceInterface(getSourceInterface());
    }

    /**
     * Get the source port
     * 
     * @return source port
     */
    public String getSourcePort() {
        return sourcePort;
    }

    /**
     * Set the source port
     * 
     * @param sourcePort source port
     */
    public void setSourcePort(String sourcePort) {
        this.sourcePort = sourcePort;
    }

    /**
     * Get the source interface
     * 
     * @return source interface
     */
    public String getSourceInterface() {
        return sourceInterface;
    }

    /**
     * Set the source interface
     * 
     * @param sourceInterface source interface
     */
    public void setSourceInterface(String sourceInterface) {
        this.sourceInterface = sourceInterface;
    }

    /**
     * Get whether the tunnel should auto-start
     * 
     * @return auto start
     */
    public boolean isAutoStart() {
        return autoStart;
    }

    /**
     * Set whether the tunnel shhould auto-start
     * 
     * @param autoStart auto start
     */
    public void setAutoStart(boolean autoStart) {
        this.autoStart = autoStart;
    }

    /**
     * Get the destination hostname or IP address
     * 
     * @return destination hostname or IP address
     */
    public String getDestinationHost() {
        return destinationHost;
    }

    /**
     * Set the destination hostname or IP address
     * 
     * @param destinationHost destination hostname or IP address
     */
    public void setDestinationHost(String destinationHost) {
        this.destinationHost = destinationHost;
    }

    /**
     * Get the destination port
     * 
     * @return destination port
     */
    public String getDestinationPort() {
        return destinationPort;
    }

    /**
     * Set the destination port.
     * 
     * @param destinationPort destination port
     */
    public void setDestinationPort(String destinationPort) {
        this.destinationPort = destinationPort;
    }

    /**
     * Get the tunnel type. This may be one of
     * {@link TransportType#REMOTE_TUNNEL_ID} or
     * {@link TransportType#LOCAL_TUNNEL_ID}.
     * 
     * @return tunnel type
     */
    public int getTunnelType() {
        return tunnelType;
    }

    /**
     * Set the tunnel type. This may be one of
     * {@link TransportType#REMOTE_TUNNEL_ID} or
     * {@link TransportType#LOCAL_TUNNEL_ID}.
     * 
     * @param tunnelType tunnel type
     */
    public void setTunnelType(int tunnelType) {
        this.tunnelType = tunnelType;
    }

    /**
     * Set the tunnel trasport. This may be one of
     * {@link TransportType#TCP_TUNNEL} or {@link TransportType#UDP_TUNNEL}.
     * 
     * @param transport transport type
     */
    public void setTransport(String transport) {
        this.transport = transport;
    }

    /**
     * Get the tunnel trasport. This may be one of
     * {@link TransportType#TCP_TUNNEL} or {@link TransportType#UDP_TUNNEL}.
     * 
     * @return transport type
     */
    public String getTransport() {
        return transport;
    }

    /**
     * Get a list of available tunnel types as {@link LabelValueBean} objects.
     * 
     * @return tunnel types
     */
    public List getTunnelTypeList() {
        return TransportType.TYPES;
    }

    /**
     * Get a list of available tunnel transports as {@link LabelValueBean}
     * objects.
     * 
     * @return tunnel types
     */
    public List getTransportList() {
        return TransportType.TRANSPORTS;
    }

    /**
     * Get the tunnel type as a string
     * 
     * @return tunnel type string
     */
    public String getTunnelTypeString() {
        return ((LabelValueBean) getTunnelTypeList().get(getTunnelType())).getLabel();
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.sslexplorer.policyframework.forms.AbstractFavoriteResourceForm#reset(org.apache.struts.action.ActionMapping,
     *      javax.servlet.http.HttpServletRequest)
     */
    public void reset(ActionMapping mapping, HttpServletRequest request) {
        super.reset(mapping, request);
        this.sourceInterface = "127.0.0.1";
        this.autoStart = false;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.sslexplorer.tabs.TabModel#getTabBundle(int)
     */
    public String getTabBundle(int idx) {
        return null;
    }
}