/*
 *  SSL-Explorer
 *
 *  Copyright (C) 2003-2006 3SP LTD. All Rights Reserved
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2 of
 *  the License, or (at your option) any later version.
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public
 *  License along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
			
package com.sslexplorer.replacementproxy;

import java.io.File;

import javax.servlet.http.HttpSessionBindingEvent;
import javax.servlet.http.HttpSessionBindingListener;

import org.apache.commons.cache.SimpleCache;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.sslexplorer.properties.Property;
import com.sslexplorer.properties.impl.systemconfig.SystemConfigKey;
import com.sslexplorer.security.User;

public class ContentCache extends SimpleCache implements HttpSessionBindingListener {

    final static Log log = LogFactory.getLog(ContentCache.class);

    ContentStash contentStash;
    User user;
    File cacheDir;

    public ContentCache(User user, File cacheDir, int maxMB, int maxObjects) {
        this(user, new ContentStash(1024 * 1024 * maxMB, maxObjects, cacheDir, 20, true));
        this.cacheDir = cacheDir;
        if (log.isInfoEnabled())
        	log.info("Created new cache at " + cacheDir.getAbsolutePath() + " for " + user.getPrincipalName() + " of capacity "
                        + contentStash.capacity() + ".");
    }

    private ContentCache(User user, ContentStash stash) {
        super(stash);
        contentStash = (ContentStash) stash;
        this.user = user;
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.servlet.http.HttpSessionBindingListener#valueBound(javax.servlet.http.HttpSessionBindingEvent)
     */
    public void valueBound(HttpSessionBindingEvent arg0) {
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.servlet.http.HttpSessionBindingListener#valueUnbound(javax.servlet.http.HttpSessionBindingEvent)
     */
    public void valueUnbound(HttpSessionBindingEvent arg0) {
        if (Property.getPropertyBoolean(new SystemConfigKey(
			"webForwards.cache.clearOnLogout"))) {
        	return;
		}
		if (log.isInfoEnabled())
			log.info("Clearing content cache for " + user.getPrincipalName());
		contentStash.clear();
		if (log.isInfoEnabled())
			log.info("Cleared content cache for " + user.getPrincipalName());
    }

}