/*
 *  SSL-Explorer
 *
 *  Copyright (C) 2003-2006 3SP LTD. All Rights Reserved
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2 of
 *  the License, or (at your option) any later version.
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public
 *  License along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
			
package com.sslexplorer.replacementproxy;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Vector;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.maverick.http.AsyncHttpMethod;
import com.maverick.http.HttpConnection;
import com.maverick.http.HttpMethod;
import com.maverick.http.HttpRequest;
import com.maverick.http.HttpResponse;
import com.maverick.util.URLUTF8Encoder;
import com.sslexplorer.boot.Util;
import com.sslexplorer.reverseproxy.ReverseProxyProxiedMethod;
import com.sslexplorer.security.SessionInfo;


/**
 * {@link AsyncHttpMethod} implementations suitable for use with replacement
 * proxy.
 * 
 * @author Brett Smith <a href="mailto: brett@3sp.com">&lt;brett@3sp.com&gt;</a>
 */
public class ReplacementProxyProxiedMethod extends HttpMethod {

	private byte[] content;
    private HttpRequest proxiedHeaders = new HttpRequest();
    private ByteArrayOutputStream out = new ByteArrayOutputStream();
    private String charsetEncoding = null;
    private boolean wwwURLEncodedParameters;
    
    static Log log = LogFactory.getLog(ReverseProxyProxiedMethod.class);
    
    /**
     * @param name
     * @param uri
     * @param parameters
     * @param session
     * @param i
     * @param string
     */
    public ReplacementProxyProxiedMethod(String name, String path, Map parameters, SessionInfo session, boolean wwwURLEncodedParameters) { // ,

        super(name, path);
        
        this.wwwURLEncodedParameters = wwwURLEncodedParameters;
        
        String key;
        Object val;
        
        for(Iterator it = parameters.keySet().iterator();it.hasNext();) {

            key = (String) it.next();
            val = parameters.get(key);

            if(val instanceof String)
                addParameter(key, (String)val);
            else if(val instanceof List) {
                // Multiple parameter values.
                List l = (List)val;
                for(Iterator it2 = l.iterator(); it2.hasNext();) {
                    addParameter(key, (String) it2.next());
                }
            }
            else if(val instanceof String[]) {
                String[] tmp = (String[]) val;
                for(int i=0;i<tmp.length;i++) {
                    addParameter(key, tmp[i]);
                }
            }
        }
    }
    
    public void setCharsetEncoding(String charsetEncoding) {
        this.charsetEncoding = charsetEncoding;
    }

    public OutputStream getOutputStream() {
        return out;
    }

    public void setContent(byte[] content, String contentType) {
        this.content = content;
        proxiedHeaders.setHeaderField("Content-Type", contentType);
        proxiedHeaders.setHeaderField("Content-Length", String.valueOf(content.length));
    }

    public void setContent(byte[] content) {
        this.content = content;
        proxiedHeaders.setHeaderField("Content-Length", String.valueOf(content.length));
    }

    public HttpRequest getProxiedRequest() {
        return proxiedHeaders;
    }

    public HttpResponse execute(HttpRequest request, HttpConnection connection) throws IOException {

        String encodedContent = "";
        /**
         * Encode parameters into content if application/x-www-form-urlencoded
         */
        if (wwwURLEncodedParameters) {
            String key;
            String value;
            Vector v;
            for (Enumeration e = getParameterNames(); e.hasMoreElements(); ) {

                key = (String) e.nextElement();
                v = getParameterValueList(key);

                for (Iterator it2 = v.iterator(); it2.hasNext();) {
                    value = (String) it2.next();
                    encodedContent += (encodedContent.length() > 0 ? "&" : "") + Util.urlEncode(key, (charsetEncoding==null ? System.getProperty("sslexplorer.urlencoding", "UTF-8") : charsetEncoding)) + "="
                    + Util.urlEncode(value, (charsetEncoding==null ? System.getProperty("sslexplorer.urlencoding", "UTF-8") : charsetEncoding));
                }
            }

            if(charsetEncoding==null)
                setContent(encodedContent.getBytes(), "application/x-www-form-urlencoded");
            else
                setContent(encodedContent.getBytes(charsetEncoding), "application/x-www-form-urlencoded");
        }

        // Setup all the proxied headers
        for (Enumeration e = proxiedHeaders.getHeaderFieldNames(); e.hasMoreElements();) {
            String header = (String) e.nextElement();
            if (header.equalsIgnoreCase("Authorization"))
                if (request.getHeaderField("Authorization") != null)
                    continue;
            String[] values = proxiedHeaders.getHeaderFields(header);
            for (int i = 0; i < values.length; i++) {
                request.addHeaderField(header, values[i]);
            }
        }

        request.performRequest(this, connection);

        if (content != null) {
            connection.getOutputStream().write(content);
        }

        return new HttpResponse(connection);
    }
    
    public String getURI() {

    	String uri = super.getURI();
    	
    	/**
    	 * LDP - Jetty decodes POST application/x-www-form-urlencoded parameters so we don't add them 
    	 * onto the URI. If we have parameters and the method IS NOT POST and content type IS NOT
    	 * application/x-www-form-urlencoded then add them to the URI as its the only place they 
    	 * could have come from. 
    	 */
    	if(getName().equals("POST") && wwwURLEncodedParameters)
    		return URLUTF8Encoder.encode(uri, false);
    	
        String encodedParams = ""; //$NON-NLS-1$
        String key;
        Vector v;
        for (Enumeration e = getParameterNames(); e.hasMoreElements(); ) {

            key = (String) e.nextElement();
            v = getParameterValueList(key);

            for (Enumeration e2 = v.elements(); e2.hasMoreElements();) {
                String value = (String) e2.nextElement();
                encodedParams += (encodedParams.length() > 0 ? "&" : "") + URLUTF8Encoder.encode(key, true) + "=" + //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
                    URLUTF8Encoder.encode(value, true);
            }
        }

        if (uri.indexOf('?') > 0 && encodedParams.length() > 0)
            return uri + "&" + encodedParams; //$NON-NLS-1$
        else
            return uri + (encodedParams.length() > 0 ? ("?" + encodedParams) : ""); //$NON-NLS-1$ //$NON-NLS-2$
    }    
}
