package com.sslexplorer.boot;

import java.io.EOFException;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.Socket;
import java.net.UnknownHostException;
import java.nio.ByteBuffer;

import javax.net.ssl.SSLContext;
import javax.net.ssl.SSLEngine;
import javax.net.ssl.SSLEngineResult;
import javax.net.ssl.SSLSession;
import javax.net.ssl.TrustManager;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.maverick.crypto.asn1.x509.X509Certificate;
import com.maverick.ssl.SSLException;
import com.maverick.ssl.SSLTransport;
import com.maverick.ssl.SSLTransportImpl;

public class SSLTransportJCE implements SSLTransport {

	private static SSLContext sslContext;
	private SSLSession session;
    private SSLEngine engine;
    private ByteBuffer dummy;
    private SSLEngineResult.HandshakeStatus hsStatus;
    private boolean initialHandshake;
    private boolean shutdown = false;

    ByteBuffer sourceBuffer;
    ByteBuffer destinationBuffer;
    SSLEngineResult res;
    static Log log = LogFactory.getLog(SSLTransportJCE.class);
    
    private InputStream rawIn;
    private OutputStream rawOut;
    
    byte[] buffer = new byte[32768];
    
    InputStream sslIn = new SSLInputStream();
    OutputStream sslOut = new SSLOutputStream();
    
    public InputStream getInputStream() {
    	return sslIn;
    }
    
    public OutputStream getOutputStream() {
    	return sslOut;
    }
    
    public void initialize(InputStream in, OutputStream out) throws IOException {
    	this.rawIn = in;
    	this.rawOut = out;
    	
//    	 Initialize SSL
        try {
            // Create an SSLEngine to use
            engine = getSSLContext().createSSLEngine();

            // Duh! we're the server
            engine.setUseClientMode(true);

            // Get the session and begin the handshake
            session = engine.getSession();
            engine.beginHandshake();
            hsStatus = engine.getHandshakeStatus();
            initialHandshake = true;

            // This is a dummy byte buffer which is used during initial
            // handshake negotiation
            dummy = ByteBuffer.allocate(0);
            destinationBuffer = ByteBuffer.allocateDirect(32768);
            
            performInitialHandshake();

        } catch(Exception ex) {
            throw new IOException(ex.getMessage());
        }    	
    }
    
    private void performInitialHandshake() throws IOException {
    	
	        /**
	         * Perform the initial handshake operation
	         */
    	
	        while(initialHandshake) {
	
	        switch (hsStatus) {
	        case FINISHED:
	            /**
	             * We have completed the handshake so lets start doing the real stuff
	             */
	            
	        	if(log.isInfoEnabled())
	        		log.info("SSL Handshake finished");
	            initialHandshake = false;
	            return;
	
	        case NEED_TASK:
	
	            /**
	             * The SSL engine needs to perform a task... do it!
	             */
	            if(log.isInfoEnabled())
	            	log.info("Performing SSLEngine task");
	
	            // Execute the tasks
	            Runnable task;
	            while ((task = engine.getDelegatedTask()) != null) {
	                task.run();
	            }
	            hsStatus = engine.getHandshakeStatus();
	            break;
	
	        case NEED_UNWRAP:
	
	            /**
	             * SSL engine wants more data to make sure we're reading from
	             * the socket
	             */
	            if(log.isInfoEnabled())
	            	log.info("Reading SSL data from raw InputStream");
	
	            if(sourceBuffer==null) {
	            	
		           	int read = rawIn.read(buffer);
		
		           	if(read==-1)
		           		throw new EOFException("Unexpected EOF whilst waiting for SSL unwrap");
		           	
		           	sourceBuffer = ByteBuffer.wrap(buffer, 0, read);
		           	
	            }
	
	           	res = engine.unwrap(sourceBuffer, destinationBuffer);
	
	           	if(!sourceBuffer.hasRemaining())
	           		sourceBuffer = null;
	           	
	           	if(res.getStatus()!=SSLEngineResult.Status.OK) {
	           		throw new IOException(res.getStatus().toString());
	           	}
	           	hsStatus = res.getHandshakeStatus();
	           	break;
	
	        case NEED_WRAP:
	
	            /**
	             * SSL engine wants to write data to the socket so make sure
	             * we can write to it
	             */
	            
	        	if(log.isInfoEnabled())
	        		log.info("Writing SSL data to raw OutputStream");
	
	        	res = engine.wrap(dummy, destinationBuffer);
	        	
	           	if(res.getStatus()!=SSLEngineResult.Status.OK) {
	           		throw new IOException(res.getStatus().toString());
	           	}
	           	
	           	hsStatus = res.getHandshakeStatus();
	        	destinationBuffer.flip();
	        	
	        	int remaining = destinationBuffer.remaining();
	        	while(remaining > 0) {
	        		destinationBuffer.get(buffer, 0, remaining);
	        		rawOut.write(buffer, 0, remaining);
	        		remaining = destinationBuffer.remaining();
	        	}
	        	
	        	destinationBuffer.compact();
	        	
	            break;
	
	        case NOT_HANDSHAKING:
	            /**
	             * This state should never be caught here
	             */
	            log.error("doHandshake has caught a NOT_HANDSHAKING state.. This is impossible!");
	        }
	    }    	
    }
    
    /**
     * Get the SSL context. If SSL isn't initialized when this is called we
     * initialize it.
     * @return SSLContext
     * @throws Exception
     */
    public static SSLContext getSSLContext() throws IOException {
        if(sslContext!=null)
            return sslContext;
        initializeSSL();
        return sslContext;
    }
	
    /**
     * Initialize SSL. If at all possible this should be called before any connection
     * attempts are made.
     *
     * @throws FileNotFoundException
     * @throws IOException
     */
    public static void initializeSSL() throws FileNotFoundException, IOException {
        try {
            sslContext = SSLContext.getInstance("TLS");
            sslContext.init(SSLKeyManager.getKeyManagerArray(), SSLTrustManager.getTrustManagerArray(), null);

        } catch (Exception ex) {
            throw new IOException("SSL initialization failed: " + ex.getMessage());
        }
    }
    
    class SSLInputStream extends InputStream {
    	
    	ByteBuffer outputData = ByteBuffer.allocateDirect(32768);
    	ByteBuffer sourceData = ByteBuffer.allocateDirect(32768); 
    	
    	SSLInputStream() {
    		outputData.flip(); // Force an empty buffer state
    	}
    	public int read() throws IOException {
    		byte[] b = new byte[1];
    		if(readData(b,0,1)==1)
    			return (int) b[0];
    		else
    			return -1;
    	}
    	
    	public int read(byte[] buf, int off, int len) throws IOException {
    		return readData(buf, off, len);
    	}
    	
        public int readData(byte[] buf, int off, int len) throws IOException {
        	
        	while(true) {

        		if(outputData.hasRemaining()) {
	        		int actualLen = Math.min(len, outputData.remaining());
	        		outputData.get(buf, off, actualLen);
	        		return actualLen;
	        	}
	        	
        		int read = -1;
        		
        		sourceData.flip();
        		
        		if(!sourceData.hasRemaining()) {
        		
	        		read = rawIn.read(buffer);
		        	
		       		if(read==-1)
		       			return -1;
		       		
		       		if(read==0)
		       			return 0;

		       		sourceData.compact();
	       			sourceData.put(buffer,0,read);
	       			sourceData.flip();
       			
        		}
       			outputData.compact();
        		
	       		res = engine.unwrap(sourceData, outputData);
	            
	       		outputData.flip();
	       		sourceData.compact();
	            
        	}
        	
        }    	
    }
    
    
    class SSLOutputStream extends OutputStream {
    	
    	ByteBuffer outputData = ByteBuffer.allocateDirect(32768);
    	
    	public void write(int b) throws IOException {
    		writeData(new byte[] { (byte)b }, 0, 1);
    	}
    	
    	public void write(byte[] buf, int off, int len) throws IOException {
    		writeData(buf, off, len);
    	}
    	
        void writeData(byte[] buf, int off, int len) throws IOException {
        	
        	ByteBuffer source = ByteBuffer.wrap(buf, off, len);
        	
        	while(source.remaining() > 0) {
    	    	res = engine.wrap(source, outputData);
    	    	
    	    	outputData.flip();
    	    	
    	    	int remaining = outputData.remaining();
    	    	while(remaining > 0) {
    	    		outputData.get(buffer, 0, remaining);
    	    		rawOut.write(buffer, 0, remaining);
    	    		remaining = outputData.remaining();
    	    	}
    	    	outputData.compact();
        	}
        }
    }
    
	public void close() throws SSLException {
	
		try {

			if(!engine.isOutboundDone())
				engine.closeOutbound();
			
			if(engine.isInboundDone())
				engine.closeInbound();
			
		} catch (Exception e) {
		} finally {
			Util.closeStream(rawIn);
			Util.closeStream(rawOut);
		}
		
	}
}
