/*
 *  SSL-Explorer
 *
 *  Copyright (C) 2003-2006 3SP LTD. All Rights Reserved
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2 of
 *  the License, or (at your option) any later version.
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public
 *  License along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
			
package com.sslexplorer.boot;

import org.jdom.Element;
import org.jdom.JDOMException;


/**
 * Extensions of {@link com.sslexplorer.boot.DefaultPropertyDefinition} 
 * that allows property defnitions to be created from XML 
 * {@link Element} objects. 
 * 
 * @author Brett Smith <a href="mailto: brett@3sp.com">&lt;brett@3sp.com&gt;</a>
 */
public class XMLPropertyDefinition extends DefaultPropertyDefinition {

    /**
     * Constructor.
     *
     */
    public XMLPropertyDefinition() {
        super();
    }

    /**
     * Constructor.
     *
     * @param type
     * @param name
     * @param typeMeta
     * @param category
     * @param defaultValue
     * @param sortOrder
     * @param hidden
     * @param validationString
     */
    public XMLPropertyDefinition(int type, String name, String typeMeta, int category, String defaultValue, int sortOrder, boolean hidden, String validationString) {
        super(type, name, typeMeta, category, defaultValue, sortOrder, hidden, validationString);
    }

    /**
     * Constructor.
     *
     * @param type
     * @param name
     * @param typeMeta
     * @param category
     * @param defaultValue
     * @param sortOrder
     * @param hidden
     */
    public XMLPropertyDefinition(int type, String name, String typeMeta, int category, String defaultValue, int sortOrder, boolean hidden) {
        super(type, name, typeMeta, category, defaultValue, sortOrder, hidden);
    }

    /**
     * Constructor.
     *
     * @param type
     * @param name
     * @param typeMeta
     * @param category
     * @param defaultValue
     * @param sortOrder
     * @param messageResourcesKey
     * @param hidden
     * @param validationString
     */
    public XMLPropertyDefinition(int type, String name, String typeMeta, int category, String defaultValue, int sortOrder, String messageResourcesKey, boolean hidden, String validationString) {
        super(type, name, typeMeta, category, defaultValue, sortOrder, messageResourcesKey, hidden, validationString);
    }

    /**
     * Constructor.
     *
     * @param type
     * @param name
     * @param typeMeta
     * @param category
     * @param defaultValue
     * @param sortOrder
     * @param messageResourcesKey
     * @param hidden
     */
    public XMLPropertyDefinition(int type, String name, String typeMeta, int category, String defaultValue, int sortOrder, String messageResourcesKey, boolean hidden) {
        super(type, name, typeMeta, category, defaultValue, sortOrder, messageResourcesKey, hidden);
    }

    /**
     * Constructor
     * 
     * @param element XML element to construct parameter from
     * @throws JDOMException on invalid XML
     */
    public XMLPropertyDefinition(Element element) throws JDOMException {
        super();
        
        // 'name' - Property name. Required 
        name = element.getAttributeValue("name");
        if (name == null || name.equals("")) {
            throw new JDOMException("Missing or empty name attribute in <parameter>");
        }
        
        // 'type' - Property type. Required
        try {
            type = Integer.parseInt(element.getAttributeValue("type"));
        } catch (Exception e) {
            throw new JDOMException("Missing or invalid type attribute in <parameter>");
        }

        // 'sortOrder' - Property sort order. Optional
        try {
            sortOrder = Integer.parseInt(element.getAttributeValue("sortOrder"));
        } catch (Exception e) {
            sortOrder = 0;
        }

        // 'typeMeta' - Property type meta info. Optional.        
        typeMeta = Util.trimmedOrBlank(element.getAttributeValue("typeMeta"));
        
        // 'validation' - Property validation string 
        validationString = element.getAttributeValue("validation");
        
        // 'hidden' - Hidden property
        hidden = "true".equalsIgnoreCase(element.getAttributeValue("hidden"));
        
        // 'category' - Property category
        try {
            String c = element.getAttributeValue("category");
            if (c != null) {
                category = Integer.parseInt(c);
            }
        } catch (Exception e) {
            throw new JDOMException("Invalid category attribute in <parameter>");
        }
        
        // 'bundle' - Property bundle
        messageResourcesKey = element.getAttributeValue("messageResourcesKey");
        messageResourcesKey = messageResourcesKey == null ? "properties" : messageResourcesKey;
        
        // 'defaultValue' - Default value
        defaultValue = element.getAttributeValue("defaultValue");
        defaultValue = defaultValue == null ? UNDEFINED_PARAMETER : defaultValue;
        defaultValue = defaultValue.replaceAll("\\\\n", "\n");
        
        // Additional type
        check();
    }

}