package com.sslexplorer.server.jetty;

import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Iterator;

import javax.servlet.UnavailableException;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.mortbay.jetty.servlet.ServletHolder;
import org.mortbay.jetty.servlet.WebApplicationContext;
import org.mortbay.jetty.servlet.WebApplicationHandler;
import org.mortbay.jetty.servlet.WebApplicationContext.Configuration;
import org.mortbay.util.Resource;
import org.mortbay.xml.XmlParser;


/**
 * Configuration implementation for precompiled JSP mappings.
 * 
 * @author Brett Smith <a href="mailto: brett@3sp.com">&lt;brett@3sp.com&gt;</a>
 */
public class JspPrecompileConfiguration implements Configuration {
	private static Log log = LogFactory.getLog(JspPrecompileConfiguration.class);
	WebApplicationContext _context;
    protected XmlParser xmlParser;

	/**
	 * Constructor.
	 *
	 */
	public JspPrecompileConfiguration() {
		super();
		xmlParser = new XmlParser();
        URL dtd=WebApplicationContext.class.getResource("/com/sslexplorer/server/jetty/jsp-mappings.dtd");
        xmlParser.redirectEntity("jsp-mappings.dtd",dtd);
        xmlParser.redirectEntity("-//3SP//JSP Mappings//EN",dtd);
	}

	public void setWebApplicationContext(WebApplicationContext context) {
		this._context = context;
	}

	public WebApplicationContext getWebApplicationContext() {
		return _context;
	}

	public void configureClassPath() throws Exception {
	}

	public void configureDefaults() throws Exception {
	}

	public void configureWebApp() throws Exception {
		Resource webInf = getWebApplicationContext().getWebInf();
		// handle any WEB-INF descriptors
		if (webInf != null && webInf.isDirectory()) {
			// do web.xml file
			Resource web = webInf.addPath("jsp-mappings.xml");
			if (!web.exists()) {
				log.info("No WEB-INF/jsp-mappings.xml in " + getWebApplicationContext().getWAR()
					+ ". Jsps will be compile on use making initial page loading slow.");
			} else {
				XmlParser.Node config = null;
				config = xmlParser.parse(web.getURL());
				initialize(config);
			}
		}
	}

	protected void initialize(XmlParser.Node config) throws ClassNotFoundException, UnavailableException {
		Iterator iter = config.iterator();
		XmlParser.Node node = null;
		while (iter.hasNext()) {
			try {
				Object o = iter.next();
				if (!(o instanceof XmlParser.Node))
					continue;
				node = (XmlParser.Node) o;
				String name = node.getTag();
				initWebXmlElement(name, node);
			} catch (ClassNotFoundException e) {
				throw e;
			} catch (Exception e) {
				log.warn("Configuration problem at " + node, e);
				throw new UnavailableException("Configuration problem");
			}
		}
	}

    protected WebApplicationHandler getWebApplicationHandler()
    {
         return _context.getWebApplicationHandler();
    }

	protected void initWebXmlElement(String element, XmlParser.Node node) throws Exception {
		if ("servlet".equals(element))
			initServlet(node);
		else if ("servlet-mapping".equals(element))
			initServletMapping(node);
	}


    protected void initServlet(XmlParser.Node node) throws ClassNotFoundException,UnavailableException,IOException,
            MalformedURLException
    {
        String name=node.getString("servlet-name",false,true);
        String className=node.getString("servlet-class",false,true);
        String jspFile=null;
        if(name==null)
            name=className;
        ServletHolder holder=getWebApplicationHandler().newServletHolder(name,className,jspFile);
        if(jspFile!=null)
            holder.setInitParameter("classpath",getWebApplicationContext().getFileClassPath());
    }

    protected void initServletMapping(XmlParser.Node node)
    {
        String name=node.getString("servlet-name",false,true);
        String pathSpec=node.getString("url-pattern",false,true);
        getWebApplicationHandler().mapPathToServlet(pathSpec,name);
    }

}
