/*
 *  SSL-Explorer
 *
 *  Copyright (C) 2003-2006 3SP LTD. All Rights Reserved
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2 of
 *  the License, or (at your option) any later version.
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public
 *  License along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
			
package com.sslexplorer.server.jetty;

import java.io.File;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.jasper.JasperException;
import org.apache.jasper.JspC;
import org.mortbay.http.ResourceCache;

import com.sslexplorer.boot.ContextHolder;
import com.sslexplorer.boot.Util;

/**
 * Precompiles Jsp for all extensions and the core.
 * 
 * NOTE This is imcomplete
 * 
 * @author Brett Smith <a href="mailto: brett@3sp.com">&lt;brett@3sp.com&gt;</a>
 */
public class JspPrecompiler {

	final static Log log = LogFactory.getLog(JspPrecompiler.class);

	/**
	 * Constructor.
	 * 
	 */
	public JspPrecompiler() {
	}


	/**
	 * Compile.
	 * 
	 * @param context context
	 * @throws Exception
	 */
	public void compile(CustomWebApplicationContext context) throws Exception {

		
		// Clean up the old web app applications directory if it exists
		File oldWebAppDir = new File("webapp" + File.separator + "WEB-INF" + File.separator + "applications");
		if(oldWebAppDir.exists()) {
			Util.delTree(oldWebAppDir);
		}
		
		/* Scan through the core and every additional resource backwards. The first place a
		 * a file exists is the one that gets compile
		 */ 
		List<String> uris = new ArrayList<String>();
		List<ResourceCache> resourceCaches = new ArrayList<ResourceCache>(context.getResourceCaches());
		Collections.reverse(resourceCaches);
		for(ResourceCache c : resourceCaches) {
			scanRoot(c.getBaseResource().getFile().getAbsolutePath(), uris);
		}		
		File coreWebapp = new File("webapp");
		scanRoot(coreWebapp.getAbsolutePath(), uris);
		
		/* Precompile */
		
		JspC jspc = new JspC() {
			public void scanFiles( File base ) throws JasperException {
				addExtension("jsp");
				addExtension("jspx");
				addExtension("jspf");
				super.scanFiles(base);
			};			
		};
		jspc.setClassPath(Util.getClassPath(context.getClassLoader()));
		jspc.setCompile(true);
		File dir = ContextHolder.getContext().getTempDirectory();		
		jspc.setOutputDir(dir.getAbsolutePath());
		jspc.setAddWebXmlMappings(false);
		
		/* For the moment, we cannot pre-compile any JSP's that exist 
		 * in plugins. This is because JspC does not know how to 
		 * find custom tag lib definitions or web.xml
		 */
		coreOnlyPrecompile(uris, coreWebapp);
		
		jspc.setJspFiles(asCommaSeparatedList(uris));
		
		jspc.setListErrors(true);
		jspc.setVerbose(9);
		jspc.execute();
	}
	
	void coreOnlyPrecompile(List<String> uris, File coreWebapp) {		
		for(String uri : new ArrayList<String>(uris)) {
			if(!uri.startsWith(coreWebapp.getAbsolutePath())) {
				uris.remove(uri);
			}
		}		
	}


	String asCommaSeparatedList(List<String> uris) {
		StringBuffer buf = new StringBuffer();
		for(String uri : uris) {
			if(buf.length() != 0) {
				buf.append(",");
			}
			buf.append(uri);
		}
		return buf.toString();
	}
	
	String asList(List<String> uris) {
		StringBuffer buf = new StringBuffer();
		for(String uri : uris) {
			if(buf.length() != 0) {
				buf.append("\n");
			}
			buf.append(uri);
		}
		return buf.toString();
	}


	void scanRoot(String root, List<String> uris) {
		List<String> paths = new ArrayList<String>();
		File dir = new File(root);
		scanDir(dir, uris, paths, "");
	}
	
	void scanDir(File file, List<String> uris, List<String> paths, String path) {
		if(file.isDirectory()) {
			File[] files = file.listFiles();
			for(int i = 0 ; i < files.length; i++) {
				String newPath = path.equals("") ? files[i].getName() : ( path + File.separator + files[i].getName() ) ;
				scanDir(files[i], uris, paths, newPath);				
			}
		}
		else {
			if(file.getName().endsWith(".jsp") || file.getName().endsWith(".jspf")) {
				String uri = file.getAbsolutePath();
				if(!paths.contains(path)) {
					uris.add(uri);
					paths.add(path);
				}
			}
		}
	}

}
