/*
 *  SSL-Explorer
 *
 *  Copyright (C) 2003-2006 3SP LTD. All Rights Reserved
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2 of
 *  the License, or (at your option) any later version.
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public
 *  License along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
			
package com.sslexplorer.agent;

import java.util.Iterator;
import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.maverick.multiplex.MultiplexedConnection;
import com.maverick.multiplex.Request;
import com.maverick.multiplex.RequestHandler;
import com.maverick.util.ByteArrayReader;
import com.maverick.util.ByteArrayWriter;
import com.sslexplorer.policyframework.LaunchSession;
import com.sslexplorer.policyframework.LaunchSessionFactory;
import com.sslexplorer.policyframework.LaunchSessionManager;
import com.sslexplorer.policyframework.Policy;
import com.sslexplorer.policyframework.PolicyDatabaseFactory;
import com.sslexplorer.policyframework.Resource;
import com.sslexplorer.policyframework.ResourceType;
import com.sslexplorer.policyframework.ResourceUtil;

public class ResourceRequestHandler implements RequestHandler {

    static Log log = LogFactory.getLog(ResourceRequestHandler.class);

	public boolean processRequest(Request request, MultiplexedConnection connection) {
		AgentTunnel agent = (AgentTunnel) connection;
		if (request.getRequestName().equals("getResources@3sp.com")) {
			try {
				if(request.getRequestData()!=null) {
					ByteArrayReader bar = new ByteArrayReader(request.getRequestData());
					int resourceTypeId = (int)bar.readInt();
					ResourceType resourceType = PolicyDatabaseFactory.getInstance().getResourceType(resourceTypeId);
					if(resourceType == null) {
						throw new Exception("Request for list of resources of an unknown type (" + resourceTypeId + ")");
					}				
					List resources = ResourceUtil.getGrantedResource(agent.getSession(), resourceType);
					Resource resource;
					ByteArrayWriter response = new ByteArrayWriter();
					response.writeInt(resources.size());
					for (Iterator it = resources.iterator(); it.hasNext();) {
						resource = (Resource) it.next();
						response.writeInt(resource.getResourceId());
						response.writeString(resource.getResourceDisplayName());
					}
					request.setRequestData(response.toByteArray());
					return true;
				}
			} catch (Exception e) {
				log.error("Failed to get resources.", e);
			}

		}
		else if (request.getRequestName().equals("setupResourceLaunch@3sp.com")) {
			try {
				if(request.getRequestData()!=null) {
					ByteArrayReader bar = new ByteArrayReader(request.getRequestData());
					int resourceTypeId = (int)bar.readInt();
					ResourceType resourceType = PolicyDatabaseFactory.getInstance().getResourceType(resourceTypeId);
					if(resourceType == null) {
						throw new Exception("Request for launch of resource of an unknown type (" + resourceTypeId + ")");
					}
					int id = (int)bar.readInt();
					Resource resource = resourceType.getResourceById(id);
					if (resource == null) {
						throw new Exception("No resource with ID " + id + " and type of " + resourceTypeId);
					}
					Policy policy = LaunchSessionManager.getLaunchRequestPolicy(null, agent.getSession(), resource);
					if (resource.sessionPasswordRequired(agent.getSession())) {
						log.error("Resource requires session password. Not yet supported.");
						return false;
					} else {
						LaunchSession launchSession = LaunchSessionFactory.getInstance().createLaunchSession(agent.getSession(),
							resource,
							policy);
						ByteArrayWriter baw = new ByteArrayWriter();
						baw.writeString(launchSession.getId());
						request.setRequestData(baw.toByteArray());
						return true;
					}
				}
			} catch (Exception e) {
				log.error("Failed to setup resource launch.", e);
				return false;
			}
		}
		return false;
	}

	public void postReply(MultiplexedConnection connection) {		
	}
}
