package com.sslexplorer.ajax;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.ajaxtags.helpers.AjaxXmlBuilder;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionMapping;

import com.sslexplorer.boot.ContextHolder;
import com.sslexplorer.core.UserDatabaseManager;
import com.sslexplorer.security.LogonControllerFactory;
import com.sslexplorer.security.Role;
import com.sslexplorer.security.SessionInfo;
import com.sslexplorer.security.UserDatabase;


/**
 * Implementation of {@link com.sslexplorer.ajax.AbstractAjaxXMLAction}
 *  that returns an XML document containing a list of roles.
 * <p>
 * Two request parameters are supported. First, the optional <b>role</b> 
 * which may contain a search string and secondly <b>maxRows</b> which is
 * an integer, defaults to 10 and determines the maximum number of results to 
 * return. If <b>account</b> is not supplied, all roles (up to the 
 * specified maximum rows) will be returned. 
 * 
 * @author Brett Smith <a href="mailto: brett@3sp.com">&lt;brett@3sp.com&gt;</a>
 */
public class ListRolesAction extends AbstractAjaxXMLAction {

    /* (non-Javadoc)
     * @see com.sslexplorer.core.actions.AuthenticatedAction#onExecute(org.apache.struts.action.ActionMapping, org.apache.struts.action.ActionForm, javax.servlet.http.HttpServletRequest, javax.servlet.http.HttpServletResponse)
     */
    protected void onAjaxRequest(ActionMapping mapping, ActionForm form, HttpServletRequest request,
                    HttpServletResponse response, AjaxXmlBuilder builder) throws Exception {
        UserDatabase udb;
        if(ContextHolder.getContext().isSetupMode()) {
            udb =  UserDatabaseManager.getInstance().getDefaultUserDatabase();
        }
        else {
            udb =  UserDatabaseManager.getInstance().getUserDatabase(LogonControllerFactory.getInstance().getUser(request).getRealm());
        }
        String role = request.getParameter("role");
        String maxRows = request.getParameter("maxRows");
        int rows = Integer.parseInt(maxRows == null ? "10" : maxRows);
        Role[] roles = udb.listAllRoles( ( role == null ? "" : role ) + "*");
        for(int i = 0 ; i < roles.length && i < rows; i++) {
            builder.addItem(roles[i].getPrincipalName(), roles[i].getPrincipalName());            
        }
    }

    /* (non-Javadoc)
     * @see com.sslexplorer.core.actions.CoreAction#getNavigationContext(org.apache.struts.action.ActionMapping, org.apache.struts.action.ActionForm, javax.servlet.http.HttpServletRequest, javax.servlet.http.HttpServletResponse)
     */
    public int getNavigationContext(ActionMapping mapping, ActionForm form, HttpServletRequest request, HttpServletResponse response) {
        return SessionInfo.MANAGEMENT_CONSOLE_CONTEXT;
    }

}
