/*
 *  SSL-Explorer
 *
 *  Copyright (C) 2003-2006 3SP LTD. All Rights Reserved
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2 of
 *  the License, or (at your option) any later version.
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public
 *  License along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
			
package com.sslexplorer.core;

import java.io.IOException;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Locale;
import java.util.Map;

import javax.servlet.ServletException;
import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import javax.servlet.http.HttpSessionBindingEvent;
import javax.servlet.http.HttpSessionBindingListener;

import org.apache.struts.Globals;
import org.apache.struts.action.Action;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;
import org.apache.struts.tiles.TilesRequestProcessor;

import com.sslexplorer.boot.ContextHolder;
import com.sslexplorer.boot.HostService;
import com.sslexplorer.boot.PropertyList;
import com.sslexplorer.boot.Util;
import com.sslexplorer.navigation.MenuTree;
import com.sslexplorer.navigation.NavigationManager;
import com.sslexplorer.properties.Property;
import com.sslexplorer.properties.impl.systemconfig.SystemConfigKey;
import com.sslexplorer.security.Constants;

/**
 * Extension of {@link org.apache.struts.tiles.TilesRequestProcessor} that
 * <strong>all</strong> requests to the struts application pass throught.
 * <p>
 * Here a map of all active session is maintained and a check is made to see if
 * the navigation menus have been constructed.
 * 
 * @author Brett Smith <a href="mailto: brett@3sp.com">&lt;brett@3sp.com&gt;</a>
 */
public class CoreRequestProcessor extends TilesRequestProcessor {

    final static Map<String, HttpSession> sessions = new HashMap<String, HttpSession>();
    static CoreRequestProcessor requestProcessor;

    /**
     * Constructor.
     * 
     */
    public CoreRequestProcessor() {
        super();
        requestProcessor = this;
    }

    /**
     * Get a static instance of the request processor.
     * 
     * @return static instance of request processor
     */
    public static CoreRequestProcessor getRequestProcessor() {
        return requestProcessor;
    }

    /**
     * Get a map of all sessions
     * 
     * @return map of all sessions
     */
    public static Map getSessions() {
        return sessions;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.apache.struts.action.RequestProcessor#process(javax.servlet.http.HttpServletRequest,
     *      javax.servlet.http.HttpServletResponse)
     */
    public void process(HttpServletRequest request, HttpServletResponse response) throws IOException, ServletException {
        HttpSession session = request.getSession();
        if (session.isNew()) {

            sessions.put(session.getId(), session);
            session.setAttribute(Constants.SESSION_HOOK, new HttpSessionBindingListener() {

                public void valueBound(HttpSessionBindingEvent arg0) {
                }

                public void valueUnbound(HttpSessionBindingEvent arg0) {
                    sessions.remove(arg0.getSession().getId());
                }
            });

            // Redirect to a valid host if not in setup mode and the feature is
            // in use
            if (!ContextHolder.getContext().isSetupMode()) {
                String action = Property.getProperty(new SystemConfigKey("webServer.invalidHostnameAction"));
                PropertyList validExternalHosts = Property.getPropertyList(new SystemConfigKey("webServer.validExternalHostnames"));
                ;
                if (validExternalHosts.size() != 0) {
                    Iterator it = validExternalHosts.iterator();
                    String host = request.getHeader("Host");
                    HostService hostService = host == null ? null : new HostService(host);
                    boolean hostOk = false;
                    String firstHost = (String) it.next();
                    if (hostService != null && !hostService.getHost().equals("")) {
                        if (hostService.getHost().startsWith("activeproxy")) {
                            int idx = hostService.getHost().indexOf(".");
                            hostService.setHost(hostService.getHost().substring(idx + 1));
                        }
                        String thisHost = firstHost;
                        do {
                            if (hostService.getHost().equals(thisHost)) {
                                hostOk = true;
                            } else {
                                if (it.hasNext()) {
                                    thisHost = (String) it.next();
                                }
                            }
                        } while (!hostOk && it.hasNext());
                    }
                    if (!hostOk) {
                        if (action.equals("redirect")) {
                            String path = (request.isSecure() ? "https" : "http") + "://" + firstHost;
                            if (ContextHolder.getContext().getPort() != 443) {
                                path += ":" + ContextHolder.getContext().getPort();
                            }
                            path += Util.getOriginalRequest(request);
                            session.invalidate();
                            response.sendRedirect(path);
                            return;
                        } else if (action.equals("error")) {
                            response.sendError(HttpServletResponse.SC_NOT_FOUND);
                            return;
                        } else if (action.equals("disconnect")) {
                            response.getOutputStream().close();
                            return;
                        }
                    }
                }
            } else {
                // We should never timeout during setup / installation
                session.setMaxInactiveInterval(Integer.MAX_VALUE);
            }

            // Set the default locale
            String locale = Property.getProperty(new SystemConfigKey("ui.defaultLanguage"));
            try {
                Cookie[] c = request.getCookies();
                if (c != null) {
                    for (int i = 0; i < c.length; i++) {
                        if (c[i].getName().equals(System.getProperty("sslexplorer.cookie", "SSLX_SSESHID") + "_LANG")) {
                            locale = c[i].getValue();
                            break;
                        }
                    }
                }
                session.setAttribute(Globals.LOCALE_KEY, locale.equals("") ? Locale.getDefault() : new Locale(locale));
            } catch (Exception e) {
                log.error("Failed to set default locale.", e);
            }

            CoreServlet.getServlet().fireCoreEvent(new NewHTTPSessionEvent(this, request, response));
        }

        /** Repopulate any messages from {@link RedirectWithMessages} */
        RedirectWithMessages.repopulate(request);

        try {
            super.process(request, response);
        } catch (ServletException se) {
            /*
             * TODO This hack is so we can redirect to the logon page if the
             * user tries to commit a form after their session has timed out.
             * Find a better way
             */
            if (se.getMessage() != null && se.getMessage().indexOf("BeanUtils.populate") != -1) {
                log.error("User probably commited a form after their session had timed out.", se);
                log.error("Cause.", se.getRootCause());
                request.getSession().getServletContext().getRequestDispatcher("/showHome.do").forward(request, response);
            } else {
                log.error("Error processing request. ", se);
                throw se;
            }
        }
    }
    
    @Override
    protected ActionForward processActionPerform(HttpServletRequest request, HttpServletResponse response, Action action, ActionForm form, ActionMapping mapping) throws IOException, ServletException {
    	    	
        ActionForward actionForward = super.processActionPerform(request, response, action, form, mapping);

        // Get any page tasks for this page
        String servletPath = request.getServletPath();
        if (servletPath.startsWith("/") && servletPath.endsWith(".do")) {
            servletPath = servletPath.substring(1, servletPath.length() - 3);
            MenuTree pageTaskMenuTree = NavigationManager.getMenuTree(PageTaskMenuTree.PAGE_TASK_MENU_TREE);
            MenuItem pageTasks = pageTaskMenuTree.getMenuItem(servletPath);
            if (pageTasks != null) {
                request.setAttribute(Constants.PAGE_TASKS, pageTaskMenuTree.rebuildMenus(pageTasks, request));
            } else {
                request.removeAttribute(Constants.PAGE_TASKS);
            }
            MenuTree toolBarMenuTree = NavigationManager.getMenuTree(ToolBarMenuTree.TOOL_BAR_MENU_TREE);
            MenuItem toolBarItems = toolBarMenuTree.getMenuItem(servletPath);
            if (toolBarItems != null) {
                request.setAttribute(Constants.TOOL_BAR_ITEMS, toolBarMenuTree.rebuildMenus(toolBarItems, request));
            } else {
                request.removeAttribute(Constants.TOOL_BAR_ITEMS);
            }
        } else {
            request.removeAttribute(Constants.PAGE_TASKS);
            request.removeAttribute(Constants.TOOL_BAR_ITEMS);
        }
        
        
        processActionMessages(request, response);
        return actionForward;
    }

    @Override
    protected boolean processValidate(HttpServletRequest request, HttpServletResponse response, ActionForm form, ActionMapping mapping) throws IOException, ServletException {
   	
    	
        boolean validated = super.processValidate(request, response, form, mapping);
        if(!validated) {
            processActionMessages(request, response);
        }
        return validated;
    }

    private static void processActionMessages(HttpServletRequest request, HttpServletResponse response) {
        if (isRunningUnitTests()) {
            addHeader(request, response, "unitTestMessages", Globals.MESSAGE_KEY);
            addHeader(request, response, "unitTestErrors", Globals.ERROR_KEY);
        }
    }

    private static void addHeader(HttpServletRequest request, HttpServletResponse response, String headerName, String key) {
        String actionMessages = getActionMessages(request, key);
        if (actionMessages.length() != 0) {
                response.setHeader(headerName, actionMessages);
        }
    }
    
    private static String getActionMessages(HttpServletRequest request, String key) {
        ActionMessages messages = (ActionMessages) request.getAttribute(key);
        if (messages != null) {
            StringBuffer buffer = new StringBuffer();
            for (Iterator itr = messages.get(); itr.hasNext();) {
                ActionMessage next = (ActionMessage) itr.next();
                buffer.append(next.getKey()).append(",");
            }
            return buffer.toString();
        }
        return "";
    }
    
    private static boolean isRunningUnitTests() {
//        String isRunningUnitTests = System.getProperty("unit.test.mode", "false");
//        return Boolean.valueOf(isRunningUnitTests);
        return true;
    }
}