/*
 *  SSL-Explorer
 *
 *  Copyright (C) 2003-2006 3SP LTD. All Rights Reserved
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2 of
 *  the License, or (at your option) any later version.
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public
 *  License along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
			
package com.sslexplorer.core;

import com.sslexplorer.boot.PropertyDefinition;
import com.sslexplorer.properties.PropertyChangeEvent;


/**
 * Default listener of {@link CoreEvent}s. 
 * 
 * @author Brett Smith <a href="mailto: brett@3sp.com">&lt;brett@3sp.com&gt;</a>
 */
public class DefaultCoreListener implements CoreListener {

    public void coreEvent(CoreEvent evt) {
        if (evt instanceof PropertyChangeEvent) {
            PropertyChangeEvent pce = (PropertyChangeEvent) evt;
            StringBuffer buf = new StringBuffer("Property ");
            buf.append(pce.getDefinition().getName());
            if (pce.getDefinition().getType() == PropertyDefinition.TYPE_PASSWORD) {
                buf.append(" changed");
            } else {
                buf.append(" from ");
                buf.append(pce.getOldValue());
                buf.append(" to ");
                buf.append(pce.getNewValue());
            }
            if (pce.getSessionInfo() != null) {
                buf.append(" (by user ");
                buf.append(pce.getSessionInfo().getUser().getPrincipalName());
                buf.append(" )");
            }
            if (CoreServlet.log.isInfoEnabled())
            	CoreServlet.log.info(buf.toString());
            if (pce.getDefinition().getName().equals("security.userDatabase")
                            || pce.getDefinition().getName().equals("webServer.protocol")
                            || pce.getDefinition().getName().equals("security.administrators")
                            || pce.getDefinition().getName().startsWith("cifs.")
                            || pce.getDefinition().getName().startsWith("jcifs.")
                            || pce.getDefinition().getName().startsWith("proxies.")) {
                CoreUtil.addMultipleGlobalWarning(GlobalWarning.MANAGEMENT_USERS,
                    new BundleActionMessage("properties", "event.configurationChangeRestartRequired.warning"));
            }
        } else if (evt.getId() >= CoreEventConstants.KEYSTORE_CREATED && evt.getId() <= CoreEventConstants.KEYSTORE_PKCS12_KEY_KEY_IMPORTED
                        && evt.getSessionInfo() != null) {
            CoreUtil.addMultipleGlobalWarning(GlobalWarning.MANAGEMENT_USERS,
                new BundleActionMessage("properties", "event.configurationChangeRestartRequired.warning"));
        }
    }
}