/*
 *  SSL-Explorer
 *
 *  Copyright (C) 2003-2006 3SP LTD. All Rights Reserved
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2 of
 *  the License, or (at your option) any later version.
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public
 *  License along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
			
package com.sslexplorer.core.stringreplacement;

import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import com.sslexplorer.boot.Replacer;
import com.sslexplorer.extensions.ExtensionDescriptor;

public class ExtensionDescriptorReplacer implements Replacer {
    
    private ExtensionDescriptor extensionDescriptor;
    private Map parameters;
    
    public ExtensionDescriptorReplacer(ExtensionDescriptor extensionDescriptor, Map parameters) {
        this.extensionDescriptor = extensionDescriptor;
        this.parameters = parameters;
    }

    public String getReplacement(Pattern pattern, Matcher matcher, String replacementPattern) {
        String match = matcher.group();
        String key = match.substring(2, match.length() - 1);
        try {
            int idx = key.indexOf(":");
            if (idx == -1) {
                throw new Exception("String replacement pattern is in incorrect format for " + key
                    + ". Must be <TYPE>:<key>");
            }
            String type = key.substring(0, idx);
            key = key.substring(idx + 1);
            if (VariableReplacement.log.isDebugEnabled())
                VariableReplacement.log.debug("Found replacement variable " + type + ":" + key);
            if (type.equalsIgnoreCase("shortcut")) {
                if (parameters == null) {
                    return null;
                }
                String val = (String) parameters.get(key);
                if (val == null) {
                    throw new Exception("Unknown key " + key + " for type " + type + ".");
                }
                return val;
            } else if (type.equalsIgnoreCase("application")) {
                if (extensionDescriptor == null) {
                    return null;
                }
                if (key.equals("id")) {
                    return extensionDescriptor.getId();
                } else if (key.equals("name")) {
                    return extensionDescriptor.getName();
                } else if (key.equals("description")) {
                    return extensionDescriptor.getDescription();
                } else if (key.equals("path")) {
                    return "/fs/apps/" + extensionDescriptor.getApplicationBundle().getId();
                } else {
                    throw new Exception("Unknown key " + key + " for type " + type + ".");
                }
            }
        } catch (Exception e) {
            VariableReplacement.log.error("A replacement failed for " + key + ".", e);
        }
        return null;
    }
    
}