/*
 *  SSL-Explorer
 *
 *  Copyright (C) 2003-2006 3SP LTD. All Rights Reserved
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2 of
 *  the License, or (at your option) any later version.
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public
 *  License along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
			
package com.sslexplorer.core.stringreplacement;

import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.sslexplorer.boot.Replacer;
import com.sslexplorer.boot.RequestHandlerRequest;

public class RequestHandlerRequestReplacer implements Replacer {
    
    final static Log log = LogFactory.getLog(RequestHandlerRequest.class);
    
    private RequestHandlerRequest request;
    
    public RequestHandlerRequestReplacer(RequestHandlerRequest request) {
        this.request = request;
    }

    public String getReplacement(Pattern pattern, Matcher matcher, String replacementPattern) {
        String match = matcher.group();
        String key = match.substring(2, match.length() - 1);
        Map parameters = request.getParameters();
        try {
            int idx = key.indexOf(":");
            if (idx == -1) {
                throw new Exception("String replacement pattern is in incorrect format for " + key
                    + ". Must be <TYPE>:<key>");
            }
            String type = key.substring(0, idx);
            key = key.substring(idx + 1);
            if (VariableReplacement.log.isDebugEnabled())
                VariableReplacement.log.debug("Found replacement variable " + type + ":" + key);
            if (type.equalsIgnoreCase("request")) {
                if (request == null) {
                    return null;
                } else if (key.equals("serverName")) {
                    return request.getHost();
                } else if (key.equals("serverPort")) {
                    return String.valueOf(request.getPort());
                } else if (key.startsWith("param.")) {
                    return (String) parameters.get(key.substring(6));
                } else if (key.startsWith("attr.")) {
                    return request.getAttribute(key.substring(5)).toString();
                } else if (key.equals("userAgent")) {
                    return request.getField("User-Agent");
                }
            }
        } catch (Exception e) {
            log.error("A replacement failed for " + key + ".", e);
        }
        return null;
    }
}