/*
 *  SSL-Explorer
 *
 *  Copyright (C) 2003-2006 3SP LTD. All Rights Reserved
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2 of
 *  the License, or (at your option) any later version.
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public
 *  License along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
			
package com.sslexplorer.extensions.actions;

import java.net.URLConnection;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.Globals;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;

import com.sslexplorer.core.BundleActionMessage;
import com.sslexplorer.core.CoreException;
import com.sslexplorer.core.CoreUtil;
import com.sslexplorer.core.GlobalWarning;
import com.sslexplorer.core.RedirectWithMessages;
import com.sslexplorer.core.actions.AuthenticatedDispatchAction;
import com.sslexplorer.extensions.ExtensionBundle;
import com.sslexplorer.extensions.ExtensionUploadHandler;
import com.sslexplorer.extensions.ExtensionBundle.ExtensionBundleStatus;
import com.sslexplorer.extensions.forms.ExtensionStoreForm;
import com.sslexplorer.extensions.store.ExtensionStore;
import com.sslexplorer.extensions.store.ExtensionStoreDescriptor;
import com.sslexplorer.policyframework.Permission;
import com.sslexplorer.policyframework.PolicyConstants;
import com.sslexplorer.policyframework.PolicyUtil;
import com.sslexplorer.properties.Property;
import com.sslexplorer.properties.impl.systemconfig.SystemConfigKey;
import com.sslexplorer.security.Constants;
import com.sslexplorer.security.SessionInfo;
import com.sslexplorer.vfs.UploadDetails;

/**
 * Implementation of
 * {@link com.sslexplorer.core.actions.AuthenticatedDispatchAction} that allows
 * an administrator to install, update and delete <i>Extensions</i>.
 * <p>
 * Extensions may be installed manually via an upload or from the <i>Extension
 * Store</i>.
 * 
 * @author Brett Smith <a href="mailto: brett@3sp.com">&lt;brett@3sp.com&gt;</a>
 */
public class ExtensionStoreDispatchAction extends AuthenticatedDispatchAction {
	final static Log log = LogFactory.getLog(ExtensionStoreDispatchAction.class);

	/**
	 * Constructor
	 */
	public ExtensionStoreDispatchAction() {
		super(PolicyConstants.EXTENSIONS_RESOURCE_TYPE, new Permission[] { PolicyConstants.PERM_CHANGE });
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.apache.struts.actions.DispatchAction#unspecified(org.apache.struts.action.ActionMapping,
	 *      org.apache.struts.action.ActionForm,
	 *      javax.servlet.http.HttpServletRequest,
	 *      javax.servlet.http.HttpServletResponse)
	 */
	public ActionForward unspecified(ActionMapping mapping, ActionForm form, HttpServletRequest request,
										HttpServletResponse response) throws Exception {
		return list(mapping, form, request, response);
	}

	/**
	 * Upload an extension
	 * 
	 * @param mapping mapping
	 * @param form form
	 * @param request request
	 * @param response response
	 * @return forward
	 * @throws Exception on any error
	 */
	public ActionForward upload(ActionMapping mapping, ActionForm form, HttpServletRequest request, HttpServletResponse response)
					throws Exception {
		PolicyUtil.checkPermissions(PolicyConstants.EXTENSIONS_RESOURCE_TYPE,
			new Permission[] { PolicyConstants.PERM_CHANGE },
			request);
		UploadDetails details = new UploadDetails("extensions",
						ExtensionUploadHandler.TYPE_EXTENSION,
						null,
						mapping.findForward("list"),
						mapping.findForward("list"),
						mapping.findForward("list"));
		int id = CoreUtil.addUpload(request.getSession(), details);
		request.setAttribute(Constants.REQ_ATTR_UPLOAD_DETAILS, new Integer(id));
		return mapping.findForward("upload");
	}

	/**
	 * Upload an extension
	 * 
	 * @param mapping mapping
	 * @param form form
	 * @param request request
	 * @param response response
	 * @return forward
	 * @throws Exception on any error
	 */
	public ActionForward refresh(ActionMapping mapping, ActionForm form, HttpServletRequest request, HttpServletResponse response)
					throws Exception {
		ExtensionStoreForm storeForm = (ExtensionStoreForm) form;
		ExtensionStoreDescriptor storeDescriptor = null;
		PolicyUtil.checkPermissions(PolicyConstants.EXTENSIONS_RESOURCE_TYPE,
			new Permission[] { PolicyConstants.PERM_CHANGE },
			request);
		ExtensionStore extensionStore = ExtensionStore.getInstance();
		ActionMessages msgs = new ActionMessages();
		try {
			extensionStore.resetExtensionStoreUpdate();
			storeDescriptor = extensionStore.getDownloadableExtensionStoreDescriptor(true);
			msgs.add(Globals.MESSAGE_KEY, new BundleActionMessage("extensions", "extensionStore.message.refreshed"));
			saveMessages(request, msgs);
		} catch (Exception e) {
			log.error("Failed to refresh extension store.", e);
			msgs.add(Globals.ERROR_KEY, new BundleActionMessage("extensions",
							"extensionStore.message.failedToRefresh",
							e.getMessage()));
			saveErrors(request, msgs);
		}
		storeForm.initialise(storeDescriptor, storeDescriptor == null ? ExtensionStore.getInstance().getExtensionBundles()
			: ExtensionStore.getInstance().getAllAvailableExtensionBundles());
		return mapping.findForward("success");
	}

	/**
	 * Remove selected extension
	 * 
	 * @param mapping mapping
	 * @param form form
	 * @param request request
	 * @param response response
	 * @return forward
	 * @throws Exception on any error
	 */
	public ActionForward remove(ActionMapping mapping, ActionForm form, HttpServletRequest request, HttpServletResponse response)
					throws Exception {
		PolicyUtil.checkPermission(PolicyConstants.EXTENSIONS_RESOURCE_TYPE,
			PolicyConstants.PERM_CHANGE,
			getSessionInfo(request).getUser());
		String id = request.getParameter("id");
		if (id == null || id.equals("")) {
			throw new Exception("No id parameter supplied.");
		}
		ExtensionBundle bundle = ExtensionStore.getInstance().getExtensionBundle(id);
		if (bundle == null) {
			throw new Exception("No application with an id of " + id);
		}
		ExtensionStore.getInstance().removeExtensionBundle(bundle);
		if (bundle.isContainsPlugin())
			CoreUtil.addMultipleGlobalWarning(GlobalWarning.MANAGEMENT_USERS, new BundleActionMessage("extensions",
							"extensionStore.message.pluginRemovedRestartRequired"));
		ActionMessages msgs = new ActionMessages();
		msgs.add(Globals.MESSAGE_KEY, new ActionMessage("extensionStore.message.applicationRemoved", bundle.getName()));
		saveMessages(request, msgs);
		return mapping.findForward("list");
	}

	/**
	 * Install selected extension
	 * 
	 * @param mapping mapping
	 * @param form form
	 * @param request request
	 * @param response response
	 * @return forward
	 * @throws Exception on any error
	 */
	public ActionForward install(ActionMapping mapping, ActionForm form, HttpServletRequest request, HttpServletResponse response)
					throws Exception {
		String id = request.getParameter("id");
		PolicyUtil.checkPermission(PolicyConstants.EXTENSIONS_RESOURCE_TYPE,
			PolicyConstants.PERM_CHANGE,
			getSessionInfo(request).getUser());
		URLConnection con = ExtensionStore.getInstance().downloadExtension(id);

		try {
			ExtensionBundle bundle = ExtensionStore.getInstance().installExtensionFromStore(id, con.getInputStream(), request);
			ExtensionStore.getInstance().licenseCheck(bundle, request, mapping.findForward("list"));
			ExtensionStore.getInstance().postInstallExtension(bundle, request);
			ActionMessages msgs = new ActionMessages();
			msgs.add(Globals.MESSAGE_KEY, new ActionMessage(false ? "extensionStore.message.applicationUpdated"
				: "extensionStore.message.applicationInstalled", bundle.getName()));
			saveMessages(request, msgs);
		} catch (CoreException ce) {
			ActionMessages errs = new ActionMessages();
			errs.add(Globals.ERROR_KEY, ce.getBundleActionMessage());
			saveErrors(request, errs);
		}
		return new RedirectWithMessages(mapping.findForward("list"), request);
	}

	/**
	 * Update selected extension
	 * 
	 * @param mapping mapping
	 * @param form form
	 * @param request request
	 * @param response response
	 * @return forward
	 * @throws Exception on any error
	 */
	public ActionForward update(ActionMapping mapping, ActionForm form, HttpServletRequest request, HttpServletResponse response)
					throws Exception {
		String id = request.getParameter("id");
		PolicyUtil.checkPermission(PolicyConstants.EXTENSIONS_RESOURCE_TYPE,
			PolicyConstants.PERM_CHANGE,
			getSessionInfo(request).getUser());
		URLConnection con = ExtensionStore.getInstance().downloadExtension(id);
		ExtensionBundle bundle = ExtensionStore.getInstance().updateExtension(id, con.getInputStream(), request);
		if (bundle.isContainsPlugin())
			CoreUtil.addMultipleGlobalWarning(GlobalWarning.MANAGEMENT_USERS, new BundleActionMessage("extensions",
							"extensionStore.message.extensionUpdatedRestartRequired"));
		ActionMessages msgs = new ActionMessages();
		msgs.add(Globals.MESSAGE_KEY, new ActionMessage(true ? "extensionStore.message.applicationUpdated"
			: "extensionStore.message.applicationInstalled", bundle.getName()));
		saveMessages(request, msgs);
		return new RedirectWithMessages(mapping.findForward("list"), request);
	}

	/**
	 * Start an extension. The 'id' request parameter must be supplied providing
	 * the extension bundle ID to stop.
	 * 
	 * @param mapping mapping
	 * @param form form
	 * @param request request
	 * @param response response
	 * @return ActionForward forward
	 * @throws Exception on any error
	 */
	public ActionForward start(ActionMapping mapping, ActionForm form, HttpServletRequest request, HttpServletResponse response)
					throws Exception {
		String id = request.getParameter("id");
		PolicyUtil.checkPermission(PolicyConstants.EXTENSIONS_RESOURCE_TYPE,
			PolicyConstants.PERM_CHANGE,
			getSessionInfo(request).getUser());
		ExtensionBundle bundle = ExtensionStore.getInstance().getExtensionBundle(id);
		if (bundle.getStatus() == ExtensionBundleStatus.ENABLED) {
			bundle.start();
		}
		if (bundle.getStatus() == ExtensionBundleStatus.STARTED) {
			bundle.activate();
		}
		ActionMessages messages = new ActionMessages();
		messages.add(Globals.MESSAGE_KEY, new ActionMessage("extensionStore.message.applicationDisable", bundle.getName()));
		saveMessages(request, messages);
		return mapping.findForward("list");
	}

	/**
	 * Stop an extension. The 'id' request parameter must be supplied providing
	 * the extension bundle ID to stop.
	 * 
	 * @param mapping mapping
	 * @param form form
	 * @param request request
	 * @param response response
	 * @return ActionForward forward
	 * @throws Exception on any error
	 */
	public ActionForward stop(ActionMapping mapping, ActionForm form, HttpServletRequest request, HttpServletResponse response)
					throws Exception {
		String id = request.getParameter("id");
		PolicyUtil.checkPermission(PolicyConstants.EXTENSIONS_RESOURCE_TYPE,
			PolicyConstants.PERM_CHANGE,
			getSessionInfo(request).getUser());
		ExtensionBundle bundle = ExtensionStore.getInstance().getExtensionBundle(id);
		bundle.stop();
		ActionMessages messages = new ActionMessages();
		messages.add(Globals.MESSAGE_KEY, new ActionMessage("extensionStore.message.applicationDisable", bundle.getName()));
		saveMessages(request, messages);
		return mapping.findForward("list");
	}

	/**
	 * Disable an extension. The 'id' request parameter must be supplied
	 * providing the extension bundle ID to stop.
	 * 
	 * @param mapping
	 * @param form
	 * @param request
	 * @param response
	 * @return ActionForward
	 * @throws Exception
	 */
	public ActionForward disable(ActionMapping mapping, ActionForm form, HttpServletRequest request, HttpServletResponse response)
					throws Exception {
		String id = request.getParameter("id");
		PolicyUtil.checkPermission(PolicyConstants.EXTENSIONS_RESOURCE_TYPE,
			PolicyConstants.PERM_CHANGE,
			getSessionInfo(request).getUser());
		ExtensionBundle bundle = ExtensionStore.getInstance().getExtensionBundle(id);
		if (!bundle.canDisable()) {
			throw new Exception("Bundle cannot be disabled.");
		}
		ExtensionStore.getInstance().disableExtension(id);
		ActionMessages messages = new ActionMessages();
		messages.add(Globals.MESSAGE_KEY, new ActionMessage("extensionStore.message.applicationDisable", bundle.getName()));
		saveMessages(request, messages);
		return unspecified(mapping, form, request, response);
	}

	/**
	 * Enable an extension. The 'id' request parameter must be supplied
	 * providing the extension bundle ID to stop.
	 * 
	 * @param mapping
	 * @param form
	 * @param request
	 * @param response
	 * @return ActionForward
	 * @throws Exception
	 */
	public ActionForward enable(ActionMapping mapping, ActionForm form, HttpServletRequest request, HttpServletResponse response)
					throws Exception {
		String id = request.getParameter("id");
		PolicyUtil.checkPermission(PolicyConstants.EXTENSIONS_RESOURCE_TYPE,
			PolicyConstants.PERM_CHANGE,
			getSessionInfo(request).getUser());
		ExtensionBundle bundle = ExtensionStore.getInstance().getExtensionBundle(id);
		if (!bundle.canEnable()) {
			throw new Exception("Bundle cannot be enabled.");
		}
		ExtensionStore.getInstance().enableExtension(id);
		ActionMessages messages = new ActionMessages();
		messages.add(Globals.MESSAGE_KEY, new ActionMessage("extensionStore.message.applicationEnable", bundle.getName()));
		saveMessages(request, messages);
		return unspecified(mapping, form, request, response);
	}

	/**
	 * List extensions
	 * 
	 * @param mapping mapping
	 * @param form form
	 * @param request request
	 * @param response response
	 * @return forward
	 * @throws Exception on any error
	 */
	public ActionForward list(ActionMapping mapping, ActionForm form, HttpServletRequest request, HttpServletResponse response)
					throws Exception {
		CoreUtil.clearFlow(request);

		if (!ExtensionStore.STORE_PREF.getBoolean("warnedAboutExtensionStoreConnect", true)) {
			if (request.getParameter("agree") != null) {
				ExtensionStore.STORE_PREF.putBoolean("warnedAboutExtensionStoreConnect", true);
				SessionInfo info = this.getSessionInfo(request);
				Property.setProperty(new SystemConfigKey("updates.automaticallyConnectToApplicationStore"),
					request.getParameter("agree"),
					info);
				Property.setProperty(new SystemConfigKey("updates.automaticallyInstallFromApplicationStore"),
					request.getParameter("agree"),
					info);
				Property.setProperty(new SystemConfigKey("updates.checkForCoreUpdates"), request.getParameter("agree"), info);
				Property.setProperty(new SystemConfigKey("updates.checkForExtensionUpdates"), request.getParameter("agree"), info);
			} else {
				return mapping.findForward("agreement");
			}
		}
		ExtensionStoreForm storeForm = (ExtensionStoreForm) form;
		ExtensionStoreDescriptor storeDescriptor = null;
		try {
			storeDescriptor = ExtensionStore.getInstance()
							.getDownloadableExtensionStoreDescriptor(request.getParameter("connect") != null || Property.getPropertyBoolean(new SystemConfigKey("updates.automaticallyConnectToApplicationStore")));
		} catch (Exception e) {
			ActionMessages errs = new ActionMessages();
			errs.add(Globals.ERROR_KEY, new ActionMessage("extensionStore.failedToContactStore", e.getMessage()));
			addErrors(request, errs);
		}
		storeForm.initialise(storeDescriptor, storeDescriptor == null ? ExtensionStore.getInstance().getExtensionBundles()
			: ExtensionStore.getInstance().getAllAvailableExtensionBundles());
		return mapping.findForward("success");
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.sslexplorer.core.actions.CoreAction#getNavigationContext(org.apache.struts.action.ActionMapping,
	 *      org.apache.struts.action.ActionForm,
	 *      javax.servlet.http.HttpServletRequest,
	 *      javax.servlet.http.HttpServletResponse)
	 */
	public int getNavigationContext(ActionMapping mapping, ActionForm form, HttpServletRequest request, HttpServletResponse response) {
		return SessionInfo.MANAGEMENT_CONSOLE_CONTEXT;
	}
}