/*
 *  SSL-Explorer
 *
 *  Copyright (C) 2003-2006 3SP LTD. All Rights Reserved
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2 of
 *  the License, or (at your option) any later version.
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public
 *  License along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
			
package com.sslexplorer.extensions.forms;

import java.util.ArrayList;
import java.util.List;

import com.sslexplorer.core.forms.CoreForm;
import com.sslexplorer.extensions.ExtensionBundle;
import com.sslexplorer.extensions.ExtensionBundleCategoryItem;
import com.sslexplorer.extensions.ExtensionBundleItem;
import com.sslexplorer.extensions.store.ExtensionStore;
import com.sslexplorer.extensions.store.ExtensionStoreDescriptor;
import com.sslexplorer.tabs.TabModel;

/**
 * Form supporting maintenance of <i>Extensions</i>.
 * 
 * @author Brett Smith <a href="mailto: brett@3sp.com">&lt;brett@3sp.com&gt;</a>
 * @author Lee David Painter <a href="mailto: brett@3sp.com">&lt;lee@3sp.com&gt;</a>
 * @author James Robinson <a href="mailto: james@3sp.com">&lt;james@3sp.com&gt;</a>
 */
public class ExtensionStoreForm extends CoreForm implements TabModel {

	ExtensionStoreDescriptor descriptor;
	private String selectedTab = ExtensionStore.INSTALLED_CATEGORY;
	private List<ExtensionBundleCategoryItem> tabbedCategories = null;

	/**
	 * Initialise the form.
	 * 
	 * @param descriptor extension store descriptor
	 * @param allAvailableExtensionBundles all extension bundles
	 */
	public void initialise(ExtensionStoreDescriptor descriptor, List<ExtensionBundle> allAvailableExtensionBundles) {
		this.descriptor = descriptor;
		this.tabbedCategories = new ArrayList<ExtensionBundleCategoryItem>();
		for (ExtensionBundle element : allAvailableExtensionBundles) {
			if (!element.isHidden()) {
				ExtensionBundleCategoryItem extensionStoreFormCategory = getExtensionStoreFormCategory(element.getCategory());
				if (extensionStoreFormCategory == null) {
					tabbedCategories.add(new ExtensionBundleCategoryItem(element.getCategory(), new ExtensionBundleItem(element,
									false)));
				} else {
					extensionStoreFormCategory.addExtensionBundleItem(new ExtensionBundleItem(element, false));
				}
			}
		}
		for (ExtensionBundleCategoryItem item : tabbedCategories) {
			item.rebuild(null);
		}
		if (tabbedCategories.size() > 0)
			this.setSelectedTab(((ExtensionBundleCategoryItem) this.tabbedCategories.get(0)).getCategory());
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.sslexplorer.tabs.TabModel#getTabCount()
	 */
	public int getTabCount() {
		return this.tabbedCategories.size();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.sslexplorer.tabs.TabModel#getTabName(int)
	 */
	public String getTabName(int idx) {
		return ((ExtensionBundleCategoryItem) this.tabbedCategories.get(idx)).getCategory();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.sslexplorer.tabs.TabModel#getTabTitle(int)
	 */
	public String getTabTitle(int i) {
		return null;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.sslexplorer.tabs.TabModel#getSelectedTab()
	 */
	public String getSelectedTab() {
		return this.selectedTab;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.sslexplorer.tabs.TabModel#setSelectedTab(java.lang.String)
	 */
	public void setSelectedTab(String selectedTab) {
		this.selectedTab = selectedTab;
	}

	/**
	 * Get the list of categories
	 * 
	 * @return categories
	 */
	public List<ExtensionBundleCategoryItem> getTabbedCategories() {
		return tabbedCategories;
	}

	/**
	 * Get the extension store descriptor.
	 * 
	 * @return extension store descriptor
	 */
	public ExtensionStoreDescriptor getDescriptor() {
		return descriptor;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.sslexplorer.tabs.TabModel#getTabBundle(int)
	 */
	public String getTabBundle(int idx) {
		return null;
	}

	private ExtensionBundleCategoryItem getExtensionStoreFormCategory(String category) {
		for (ExtensionBundleCategoryItem element : tabbedCategories) {
			if (element.getCategory().equals(category)) {
				return element;
			}
		}
		return null;
	}
}