/*
 *  SSL-Explorer
 *
 *  Copyright (C) 2003-2006 3SP LTD. All Rights Reserved
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2 of
 *  the License, or (at your option) any later version.
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public
 *  License along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
			
package com.sslexplorer.extensions.store;

import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLConnection;
import java.text.DateFormat;
import java.util.Date;
import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.sslexplorer.boot.Util;
import com.sslexplorer.boot.VersionInfo;
import com.sslexplorer.extensions.ExtensionBundle;
import com.sslexplorer.properties.Property;
import com.sslexplorer.properties.impl.systemconfig.SystemConfigKey;

/**
 */
public final class UpdateChecker extends Thread {

    private static final Log log = LogFactory.getLog(UpdateChecker.class);
    private static final int ONE_DAY_MILLIS = 1000 * 60 * 60 * 24;
    private static final int ONE_HOUR_MILLIS = 1000 * 60 * 60;
    private static final String HOME_URL = "http://3sp.com/";
    private static final String AVAILABLE_VERSION_URL = HOME_URL + "getAvailableVersion.do";
    
    private VersionInfo.Version availableVersion;
    private List<ExtensionBundle> updateable;
    private boolean running = false;

    /**
     * Default consuctor
     */
    public UpdateChecker() {
        super("UpdateThread");
        setPriority(Thread.MIN_PRIORITY);
    }

    /**
     * Intialise the update checker. If the option <i>updates.checkForCoreUpdates</i> is enabled, 
     * the currently available core version will be checked for. If the option <i>updates.checkForExtensionUpdates</i> 
     * is enabled, a check will be made for any available updates. If  there are some,
     * and the option <i>updates.automaticallyInstallFromApplicationStore</i> is enabled they
     * will be installed.   
     * 
     * @throws Exception on any error check for available version of installing extensions
     */
    public void initialise() throws Exception {
        if (Property.getPropertyBoolean(new SystemConfigKey("updates.checkForCoreUpdates"))) {
            checkAvailableVersion();
        }        

        /*
        if (Property.getPropertyBoolean(new SystemConfigKey("updates.checkForExtensionUpdates"))) {        
            Collection<ExtensionBundle> extensionBundles = getDownloadableExtensionBundles();
            
            // There is no need to download differently versioned plugins if we
            // are running in development mode
            if(!"true".equals(System.getProperty("sslexplorer.useDevConfig"))) {
                performMandatoryUpgrades(extensionBundles);        	
            }
            setAvailableExtensions(extensionBundles);
        }
        */
    }
    
    /*
    private static void performMandatoryUpgrades(Collection<ExtensionBundle> extensionBundles) throws Exception {
        for (Iterator itr = extensionBundles.iterator(); itr.hasNext();) {
            ExtensionBundle bundle = (ExtensionBundle) itr.next();
            if(performMandatoryUpgrade(bundle)) {
                itr.remove();
            }
        }
    }
    */
    
    /*
    private static boolean performMandatoryUpgrade(ExtensionBundle bundle) throws ExtensionException, IOException {
        ExtensionStore store = ExtensionStore.getInstance();
        if(store.isExtensionBundleLoaded(bundle.getId())) {
            ExtensionBundle currentExtensionBundle = store.getExtensionBundle(bundle.getId());
            boolean versionChanged = !bundle.getVersion().equals(currentExtensionBundle.getVersion());
            if(!versionChanged)
                return false;
            
            if(isAutomaticUpdateEnabled()) { 
                performUpgrade(bundle);
                return true;
            }
            else if (bundle.isMandatoryUpdate()) {
                store.systemDisableExtension(bundle.getId());
                return false;
            }
        }
        return false;
    }
    */
    
    /*
    private static void performUpgrade(ExtensionBundle bundle) throws ExtensionException, IOException {   
        ExtensionStore.getInstance().installExtensionFromStore(bundle.getId(), null);
    }
    */
    
    /*
    private static boolean isAutomaticUpdateEnabled() {
        boolean automaticInstall = Property.getPropertyBoolean(new SystemConfigKey("updates.automaticallyInstallFromApplicationStore"));
        return automaticInstall;
    }
    */
    
    public void run() {
        running = true;
        try {
            while(running) {
                if (log.isInfoEnabled())
                    log.info("Checking for SSL-Explorer and Extension updates");
                
                int checkAgainIn = ONE_DAY_MILLIS;
                sleep(checkAgainIn);
                if (!running)
                    break;
                try {
                    // Check for core update
                    if (Property.getPropertyBoolean(new SystemConfigKey("updates.checkForCoreUpdates"))) {
                        checkAvailableVersion();
                    }
                    // Check for extensions update
                    /*
                    if (Property.getPropertyBoolean(new SystemConfigKey("updates.checkForExtensionUpdates"))) {
                        checkAvailableExtensions();
                    }
                    */
                } catch (Exception e) {
                    checkAgainIn = ONE_HOUR_MILLIS;
                    log.error("Failed to check available version / extension updates. Will try again in 1 hour", e);
                }
            } 
        } catch (Exception e) {
            if (log.isDebugEnabled())
                log.debug("Update Checker exception thrown.", e);
        }
    }
    
    protected void end(){
        running = false;
        synchronized (this) {
            interrupt();
        }
    }
    
    private static void sleep(int checkAgainIn) throws InterruptedException {
        if (log.isInfoEnabled())
            log.info("Finished checking for updates / feeds, next check will occur at " + getDateAsString(checkAgainIn));
        Thread.sleep(checkAgainIn);
    }
    
    private static String getDateAsString(int checkAgainIn) {
        return DateFormat.getDateTimeInstance().format(new Date(System.currentTimeMillis() + checkAgainIn));
    }
    
    private void checkAvailableVersion() throws IOException {
        if (log.isInfoEnabled())
            log.info("Checking for SSL-Explorer core updates.");

        URL availableVersionURL = getAvailableVersionURL();
        if (availableVersionURL != null) {
            if (log.isInfoEnabled())
                log.info("Loading available version from " + availableVersionURL.toExternalForm());
            
            URLConnection conx = availableVersionURL.openConnection();
            conx.setConnectTimeout(ExtensionStore.CONNECT_TIMEOUT);
            conx.setReadTimeout(ExtensionStore.READ_TIMEOUT);
            
            InputStream inputStream = null;
            try {
                inputStream = conx.getInputStream();
                BufferedReader reader = new BufferedReader(new InputStreamReader(inputStream));
                availableVersion = new VersionInfo.Version(reader.readLine());
            } finally {
                Util.closeStream(inputStream);
            }
            if (log.isInfoEnabled())
                log.info("Version " + availableVersion + " is the latest available.");
        }
    }
    
    private static URL getAvailableVersionURL() {
        try {
            return new URL(getVersionLocation(AVAILABLE_VERSION_URL));
        } catch (MalformedURLException murle) {
            try {
                return new File(getVersionLocation("")).toURL();
            } catch (MalformedURLException e) {
                log.error("Invalid available version action location specified in system property sslexplorer.availableVersion.location, '" 
                    + getVersionLocation("") + "'. Must be either a URL or the file path of the available version document.");
            }
        }
        return null;
    }
    
    private static String getVersionLocation(String defaultValue) {
        return System.getProperty("sslexplorer.availableVersion.location", defaultValue);
    }
    /*
    
    private void checkAvailableExtensions() throws Exception {
        if (log.isInfoEnabled())
            log.info("Checking for available extension updates.");
        Collection<ExtensionBundle> extensionBundles = getDownloadableExtensionBundles();
        setAvailableExtensions(extensionBundles);
    }
    */
    
    /*
    private void setAvailableExtensions(Collection<ExtensionBundle> extensionBundles) throws Exception {
        updateable = new ArrayList<ExtensionBundle>();
        ExtensionStore extensionStore = ExtensionStore.getInstance();
        for (ExtensionBundle bundle : extensionBundles) {
            if (extensionStore.isExtensionLoaded(bundle.getId())) {
                ExtensionBundle installed = extensionStore.getExtensionBundle(bundle.getId());
                if (bundle.getVersion().compareTo(installed.getVersion()) > 0) {
                    updateable.add(bundle);
                }
            }
        }
    }
    */
    
    /*
    
    @SuppressWarnings("unchecked")
    private static Collection<ExtensionBundle> getDownloadableExtensionBundles() throws IOException, JDOMException {
        ExtensionStore extensionStore = ExtensionStore.getInstance();
        ExtensionStoreDescriptor descriptor = extensionStore.getDownloadableExtensionStoreDescriptor(true);     
        return descriptor.getExtensionBundles();
    }
    */

    /**
     * Check what version there is a new version of the SSL-Explorer core
     * available. This will only be actually retrieved from the 3SP site after a
     * restart or if a day has elapsed since the last update.
     * @return last available core version
     */
    public VersionInfo.Version getAvailableCoreVersion() {
        return availableVersion;
    }

    /**
     * Returns a list of {@link String}s contain extension bundles names that
     * may be updated.
     * @return list of updateable extensions
     */
    public List<ExtensionBundle> getUpdateableExtensions() {
        return updateable;
    }
}