/*
 *  SSL-Explorer
 *
 *  Copyright (C) 2003-2006 3SP LTD. All Rights Reserved
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2 of
 *  the License, or (at your option) any later version.
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public
 *  License along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
			
package com.sslexplorer.notification.agent;

import java.io.IOException;
import java.util.Iterator;
import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.maverick.multiplex.Request;
import com.maverick.util.ByteArrayWriter;
import com.sslexplorer.agent.AgentTunnel;
import com.sslexplorer.agent.DefaultAgentManager;
import com.sslexplorer.notification.Message;
import com.sslexplorer.notification.MessageSink;
import com.sslexplorer.notification.Notifier;
import com.sslexplorer.notification.Recipient;
import com.sslexplorer.security.LogonControllerFactory;
import com.sslexplorer.security.SessionInfo;

public class AgentMessageSink implements MessageSink {
    final static Log log = LogFactory.getLog(AgentMessageSink.class);
    private Notifier notifier;

    public AgentMessageSink() {
        super();
    }

    public void start(Notifier notifier) throws Exception {
        this.notifier = notifier;
    }

    public void stop() throws Exception {
    }

    public boolean send(Message message) throws Exception {
        //
        int sent = 0;
        List l = notifier.getFullRecipientListAsUsers(message.getRecipients());
        for (Iterator i = l.iterator(); i.hasNext();) {
            Recipient r = (Recipient) i.next();
            List sessionInfo = LogonControllerFactory.getInstance().getSessionInfo(r.getRecipientAlias(), SessionInfo.UI);
            if (sessionInfo != null) {
                for (Iterator j = sessionInfo.iterator(); j.hasNext();) {
                    SessionInfo info = (SessionInfo) j.next();
                    
                    ByteArrayWriter msg = new ByteArrayWriter();
                    msg.writeString(message.getSubject());
                    msg.writeInt(0);
                    msg.writeString(message.getContent());
                    
                    if(DefaultAgentManager.getInstance().hasActiveAgent(info) && info.getUser().getPrincipalName().equals(r.getRecipientAlias())) {
                    	try {
							Request request = new Request("agentMessage@3sp.com", msg.toByteArray());
							AgentTunnel tunnel = DefaultAgentManager.getInstance().getAgentBySession(info);
							if(tunnel!=null) {
								tunnel.sendRequest(request, false, 0);
								sent++;
							}
						} catch (IOException e) {
							log.error("Failed to send message to agent. Did it disconnect before we could send it?", e);
						}
                    }

                }
            }
        }
        return sent > 0;
    }

    public String getName() {
        return "AGENT";
    }

    public String getShortNameKey() {
        return "notification.vpnClient.shortName";
    }

    public String getDescriptionKey() {
        return "notification.vpnClient.description";
    }

    public String getBundle() {
        return "setup";
    }
}
