package com.sslexplorer.policyframework;

import java.util.Calendar;

import com.sslexplorer.security.SessionInfo;

/**
 * Resource interface, defines common methods for all resources.
 * 
 * @author James D Robinson <a href="mailto:james@3sp.com">&lt;james@3sp.com&gt;</a>
 * 
 * 
 */
public interface Resource extends Comparable {

    /**
     * Constant for the max resource name length
     */
    public final int MAX_RESOURCE_NAME_LENGTH = 32;

    /**
     * @return Rsource ID
     */
    public int getResourceId();

    /**
     * @return ResourceType
     */
    public ResourceType getResourceType();

    /**
     * @return Resource name
     */
    public String getResourceName();

    /**
     * @return String
     */
    public String getResourceDisplayName();

    /**
     * @return Resource description
     */
    public String getResourceDescription();

    /**
     * @param name Resource Name
     */
    public void setResourceName(String name);

    /**
     * @param description Resource description.
     */
    public void setResourceDescription(String description);

    /**
     * @return Date created
     */
    public Calendar getDateCreated();

    /**
     * @return Date changed
     */
    public Calendar getDateAmended();

    /**
     * @param date Date changed
     */
    public void setDateAmended(Calendar date);

    /**
     * @param sessionInfo
     * @return Resource requires session password.
     */
    public boolean sessionPasswordRequired(SessionInfo sessionInfo);

    /**
     * @return do resource parameters require session passwod.
     */
    public boolean paramsRequirePassword();

    /**
     * @return the realm ID the resource is in.
     */
    public int getRealmID();
}
