/*
 *  SSL-Explorer
 *
 *  Copyright (C) 2003-2006 3SP LTD. All Rights Reserved
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2 of
 *  the License, or (at your option) any later version.
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public
 *  License along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
			
package com.sslexplorer.policyframework.actions;

import java.util.Iterator;
import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.Globals;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;

import com.sslexplorer.core.forms.CoreForm;
import com.sslexplorer.input.MultiSelectDataSource;
import com.sslexplorer.policyframework.AccessRight;
import com.sslexplorer.policyframework.AccessRights;
import com.sslexplorer.policyframework.DelegatedPoliciesDataSource;
import com.sslexplorer.policyframework.NoPermissionException;
import com.sslexplorer.policyframework.Permission;
import com.sslexplorer.policyframework.PolicyConstants;
import com.sslexplorer.policyframework.PolicyDatabaseFactory;
import com.sslexplorer.policyframework.Resource;
import com.sslexplorer.policyframework.ResourceUtil;
import com.sslexplorer.policyframework.forms.AbstractResourceForm;
import com.sslexplorer.policyframework.forms.AccessRightsForm;
import com.sslexplorer.security.LogonControllerFactory;
import com.sslexplorer.security.SessionInfo;


/**
 * Implementation of a {@link com.sslexplorer.policyframework.actions.AbstractResourceDispatchAction}
 * that allows viewing and editing of a single <i>Resource Permission</i>.
 * 
 * @author Brett Smith <a href="mailto: brett@3sp.com">&lt;brett@3sp.com&gt;</a>
 */
public class AccessRightsDispatchAction extends AbstractResourceDispatchAction {

    final static Log log = LogFactory.getLog(AccessRightsDispatchAction.class);
    
    /**
     * Constructor
     */
    public AccessRightsDispatchAction() {
        super(PolicyConstants.ACCESS_RIGHTS_RESOURCE_TYPE, null, null, null, null, null);
    }

    /* (non-Javadoc)
     * @see com.sslexplorer.policyframework.actions.AbstractResourceDispatchAction#createResource(org.apache.struts.action.ActionMapping, org.apache.struts.action.ActionForm, javax.servlet.http.HttpServletRequest, javax.servlet.http.HttpServletResponse)
     */
    public Resource createResource(ActionMapping mapping, ActionForm form, HttpServletRequest request, HttpServletResponse response)
                    throws Exception {
        throw new Exception("Create not supported.");
    }

    /**
     * Add the selected permission to the list
     * 
     * @param mapping mapping
     * @param form form
     * @param request request
     * @param response response
     * @return forward
     * @throws Exception on any error
     */
    public ActionForward addSelected(ActionMapping mapping, ActionForm form, HttpServletRequest request,
                    HttpServletResponse response) throws Exception {
        AccessRightsForm drpf = (AccessRightsForm) form;
        String[] sel = drpf.getSelectedAvailablePermissions();
        List l = drpf.getResourceTypePermissions();
        for (int i = 0; i < sel.length; i++) {
            l.add(new AccessRight(drpf.getResourceType(), drpf.getResourceType().getPermission(
                Integer.parseInt(sel[i]))));
        }
        drpf.rebuild();
        return display(mapping, form, request, response);
    }

    /**
     * Add the selected permission to the list
     * 
     * @param mapping mapping
     * @param form form
     * @param request request
     * @param response response
     * @return forward
     * @throws Exception on any error
     */
    public ActionForward removeSelected(ActionMapping mapping, ActionForm form, HttpServletRequest request,
                    HttpServletResponse response) throws Exception {
        AccessRightsForm drpf = (AccessRightsForm) form;
        String[] sel = drpf.getSelectedResourceTypePermissions();
        if(sel != null) {
            List l = drpf.getResourceTypePermissions();
            for (int i = 0; i < sel.length; i++) {
                int idx = sel[i].indexOf('-');
                int resourceTypeId = Integer.parseInt(sel[i].substring(0, idx));
                int permissionId = Integer.parseInt(sel[i].substring(idx + 1));
                AccessRight sp = null;
                for (Iterator j = l.iterator(); j.hasNext() && sp == null;) {
                    AccessRight p = (AccessRight) j.next();
                    if (p.getResourceType().getResourceTypeId() == resourceTypeId && p.getPermission().getId() == permissionId) {
                        sp = p;
                    }
                }
                if (sp != null) {
                    l.remove(sp);
                }
            }
            drpf.rebuild();
        }
        return display(mapping, form, request, response);
    }

    /**
     * Change the selected resource type
     * 
     * @param mapping mapping
     * @param form form
     * @param request request
     * @param response response
     * @return forward
     * @throws Exception on error
     */
    public ActionForward selectResourceType(ActionMapping mapping, ActionForm form, HttpServletRequest request,
                    HttpServletResponse response) throws Exception {
        AccessRightsForm drpf = (AccessRightsForm) form;
        drpf.setResourceType(PolicyDatabaseFactory.getInstance().getResourceType(drpf.getSelectedResourceType()));
        drpf.rebuild();
        return display(mapping, form, request, response);
    }

    /* (non-Javadoc)
     * @see com.sslexplorer.policyframework.actions.AbstractResourceDispatchAction#commit(org.apache.struts.action.ActionMapping, org.apache.struts.action.ActionForm, javax.servlet.http.HttpServletRequest, javax.servlet.http.HttpServletResponse)
     */
    public ActionForward commit(ActionMapping mapping, ActionForm form, HttpServletRequest request, HttpServletResponse response)
                    throws Exception {
        ActionMessages msgs = new ActionMessages();
        msgs.add(Globals.MESSAGE_KEY, new ActionMessage("editAccessRights.message.saved"));
        this.addMessages(request, msgs);
        return super.commit(mapping, form, request, response);
    }

    /* (non-Javadoc)
     * @see com.sslexplorer.core.actions.CoreAction#getNavigationContext(org.apache.struts.action.ActionMapping, org.apache.struts.action.ActionForm, javax.servlet.http.HttpServletRequest, javax.servlet.http.HttpServletResponse)
     */
    public int getNavigationContext(ActionMapping mapping, ActionForm form, HttpServletRequest request, HttpServletResponse response) {
        return SessionInfo.MANAGEMENT_CONSOLE_CONTEXT;
    }

    /* (non-Javadoc)
     * @see com.sslexplorer.policyframework.actions.AbstractResourceDispatchAction#checkValid(com.sslexplorer.policyframework.Resource, com.sslexplorer.policyframework.Permission, org.apache.struts.action.ActionMapping, com.sslexplorer.policyframework.forms.AbstractResourceForm, javax.servlet.http.HttpServletRequest)
     */
    protected void checkValid(Resource r, Permission[] permissions, ActionMapping mapping, AbstractResourceForm form,
                    HttpServletRequest request) throws NoPermissionException {
        ResourceUtil.checkAccessRightsValid((AccessRights)r, this.getSessionInfo(request), ((CoreForm) form).getActionTarget());
    }

    /* (non-Javadoc)
     * @see com.sslexplorer.policyframework.actions.AbstractResourceDispatchAction#createAvailablePoliciesDataSource(com.sslexplorer.policyframework.Resource, org.apache.struts.action.ActionMapping, org.apache.struts.action.ActionForm, javax.servlet.http.HttpServletRequest, javax.servlet.http.HttpServletResponse)
     */
    protected MultiSelectDataSource createAvailablePoliciesDataSource(Resource resource, ActionMapping mapping, ActionForm form,
                    HttpServletRequest request, HttpServletResponse response) throws Exception {
        return new DelegatedPoliciesDataSource(null, null, ((AccessRights) resource).getAccessRightsClass(), LogonControllerFactory.getInstance().getUser(request));
    }

    @Override
    protected void doUpdate(ActionMapping mapping, ActionForm form, HttpServletRequest request, HttpServletResponse response) throws Exception {
        super.doUpdate(mapping, form, request, response);
        // we now need to rebuild any menus, as more or less could be visable.
        LogonControllerFactory.getInstance().applyMenuItemChanges(request);
    }

}