/*
 *  SSL-Explorer
 *
 *  Copyright (C) 2003-2006 3SP LTD. All Rights Reserved
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2 of
 *  the License, or (at your option) any later version.
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public
 *  License along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
			
package com.sslexplorer.policyframework.actions;

import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;

import com.sslexplorer.core.forms.CoreForm;
import com.sslexplorer.navigation.MenuTree;
import com.sslexplorer.policyframework.AccessRights;
import com.sslexplorer.policyframework.AccessRightsItem;
import com.sslexplorer.policyframework.NoPermissionException;
import com.sslexplorer.policyframework.Permission;
import com.sslexplorer.policyframework.PolicyConstants;
import com.sslexplorer.policyframework.PolicyDatabaseFactory;
import com.sslexplorer.policyframework.Resource;
import com.sslexplorer.policyframework.ResourceUtil;
import com.sslexplorer.policyframework.forms.AbstractResourcesForm;
import com.sslexplorer.policyframework.forms.AccessRightsListForm;
import com.sslexplorer.security.LogonControllerFactory;
import com.sslexplorer.security.SessionInfo;
import com.sslexplorer.security.User;

/**
 * Implementation of a
 * {@link com.sslexplorer.policyframework.actions.AbstractResourcesDispatchAction}
 * that allows listing and maintenance of <i>Resource Permissions</i>.
 * 
 * @author Brett Smith <a href="mailto: brett@3sp.com">&lt;brett@3sp.com&gt;</a>
 */
public class AccessRightsListDispatchAction extends AbstractResourcesDispatchAction {

    /**
     * Constructor
     */
    public AccessRightsListDispatchAction() {
        super(PolicyConstants.ACCESS_RIGHTS_RESOURCE_TYPE, null, null, null, null, null);
    }

    /* (non-Javadoc)
     * @see org.apache.struts.actions.DispatchAction#unspecified(org.apache.struts.action.ActionMapping, org.apache.struts.action.ActionForm, javax.servlet.http.HttpServletRequest, javax.servlet.http.HttpServletResponse)
     */
    public ActionForward unspecified(ActionMapping mapping, ActionForm form, HttpServletRequest request,
                    HttpServletResponse response) throws Exception {
        AccessRightsListForm accessRightsListForm = (AccessRightsListForm) form;
        User user = LogonControllerFactory.getInstance().getUser(request);
        List resourcePermissions = PolicyDatabaseFactory.getInstance().getAccessRights(user.getRealm().getRealmID());
        accessRightsListForm.initialize(resourcePermissions, AccessRightsItem.class, request.getSession());
        return super.unspecified(mapping, form, request, response);
    }

    /* (non-Javadoc)
     * @see com.sslexplorer.policyframework.actions.AbstractResourcesDispatchAction#checkValid(com.sslexplorer.policyframework.Resource, com.sslexplorer.policyframework.Permission, org.apache.struts.action.ActionMapping, com.sslexplorer.policyframework.forms.AbstractResourcesForm, javax.servlet.http.HttpServletRequest)
     */
    protected void checkValid(Resource r, Permission[] permission, ActionMapping mapping, AbstractResourcesForm form,
                    HttpServletRequest request) throws NoPermissionException {
        ResourceUtil.checkAccessRightsValid((AccessRights) r, getSessionInfo(request), ((CoreForm) form).getActionTarget());
    }

    /* (non-Javadoc)
     * @see com.sslexplorer.core.actions.CoreAction#getNavigationContext(org.apache.struts.action.ActionMapping, org.apache.struts.action.ActionForm, javax.servlet.http.HttpServletRequest, javax.servlet.http.HttpServletResponse)
     */
    public int getNavigationContext(ActionMapping mapping, ActionForm form, HttpServletRequest request, HttpServletResponse response) {
        return SessionInfo.MANAGEMENT_CONSOLE_CONTEXT;
    }

    @Override
    protected void doRemove(ActionMapping mapping, ActionForm form, HttpServletRequest request, HttpServletResponse response) throws Exception {
        super.doRemove(mapping, form, request, response);
        // we now need to rebuild any menus, as more or less could be visable.
        LogonControllerFactory.getInstance().applyMenuItemChanges(request);
    }   
}