/*
 *  SSL-Explorer
 *
 *  Copyright (C) 2003-2006 3SP LTD. All Rights Reserved
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2 of
 *  the License, or (at your option) any later version.
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public
 *  License along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
			
package com.sslexplorer.policyframework.wizards.actions;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;

import com.sslexplorer.core.CoreUtil;
import com.sslexplorer.policyframework.AccessRight;
import com.sslexplorer.policyframework.Permission;
import com.sslexplorer.policyframework.wizards.forms.AccessRightsPermissionsForm;
import com.sslexplorer.security.SessionInfo;
import com.sslexplorer.wizard.AbstractWizardSequence;
import com.sslexplorer.wizard.actions.AbstractWizardAction;
import com.sslexplorer.wizard.forms.AbstractWizardForm;

/**
 * <p>
 * Action for selecting access rights permissions
 * 
 * @author James D Robinson <a href="mailto:james@3sp.com">&lt;james@3sp.com&gt;</a>
 * 
 */
public class AccessRightsPermissionsAction extends AbstractWizardAction {

    /**
     * Constructor
     */
    public AccessRightsPermissionsAction() {
        super();
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.sslexplorer.core.actions.CoreAction#getNavigationContext(org.apache.struts.action.ActionMapping,
     *      org.apache.struts.action.ActionForm,
     *      javax.servlet.http.HttpServletRequest,
     *      javax.servlet.http.HttpServletResponse)
     */
    public int getNavigationContext(ActionMapping mapping, ActionForm form, HttpServletRequest request, HttpServletResponse response) {
        return SessionInfo.MANAGEMENT_CONSOLE_CONTEXT;
    }

    /**
     * @param mapping
     * @param form
     * @param request
     * @param response
     * @return ActionForward
     * @throws Exception
     */
    public ActionForward selectResourceType(ActionMapping mapping, ActionForm form, HttpServletRequest request,
                    HttpServletResponse response) throws Exception {
        applyToSequence(mapping, (AbstractWizardForm) form, request, response);
        return unspecified(mapping, form, request, response);
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.sslexplorer.wizard.actions.AbstractWizardAction#unspecified(org.apache.struts.action.ActionMapping,
     *      org.apache.struts.action.ActionForm,
     *      javax.servlet.http.HttpServletRequest,
     *      javax.servlet.http.HttpServletResponse)
     */
    public ActionForward unspecified(ActionMapping mapping, ActionForm form, HttpServletRequest request,
                    HttpServletResponse response) throws Exception {
        CoreUtil.addRequiredFieldMessage(this, request);
        return super.unspecified(mapping, form, request, response);
    }

    /**
     * @param mapping
     * @param form
     * @param request
     * @param response
     * @return ActionForward
     * @throws Exception
     */
    public ActionForward addSelected(ActionMapping mapping, ActionForm form, HttpServletRequest request,
                    HttpServletResponse response) throws Exception {
        applyToSequence(mapping, (AbstractWizardForm) form, request, response);
        AccessRightsPermissionsForm drpf = (AccessRightsPermissionsForm) form;
        String[] sel = drpf.getSelectedAvailablePermissions();
        AbstractWizardSequence seq = getWizardSequence(request);
        List l = (List) seq.getAttribute(AccessRightsPermissionsForm.ATTR_RESOURCE_TYPE_PERMISSIONS, new ArrayList());
        for (int i = 0; i < sel.length; i++) {
            // Make sure the selected permission was one that the current user
            // is actually allowed to use
            Permission perm = drpf.getResourceType().getPermission(Integer.parseInt(sel[i]));
            if (!drpf.getAvailablePermissions().contains(perm)) {
                throw new Exception("Permission denied.");
            }
            l.add(new AccessRight(drpf.getResourceType(), perm));
        }
        seq.putAttribute(AccessRightsPermissionsForm.ATTR_RESOURCE_TYPE_PERMISSIONS, l);
        return unspecified(mapping, form, request, response);
    }

    /**
     * @param mapping
     * @param form
     * @param request
     * @param response
     * @return ActionForward
     * @throws Exception
     */
    public ActionForward removeSelected(ActionMapping mapping, ActionForm form, HttpServletRequest request,
                    HttpServletResponse response) throws Exception {
        applyToSequence(mapping, (AbstractWizardForm) form, request, response);
        AccessRightsPermissionsForm drpf = (AccessRightsPermissionsForm) form;
        AbstractWizardSequence seq = getWizardSequence(request);
        String[] sel = drpf.getSelectedResourceTypePermissions();
        if (sel != null) {
            List l = (List) seq.getAttribute(AccessRightsPermissionsForm.ATTR_RESOURCE_TYPE_PERMISSIONS, new ArrayList());
            for (int i = 0; i < sel.length; i++) {
                int idx = sel[i].indexOf('-');
                int resourceTypeId = Integer.parseInt(sel[i].substring(0, idx));
                int permissionId = Integer.parseInt(sel[i].substring(idx + 1));
                AccessRight sp = null;
                for (Iterator j = l.iterator(); j.hasNext() && sp == null;) {
                    AccessRight p = (AccessRight) j.next();
                    if (p.getResourceType().getResourceTypeId() == resourceTypeId && p.getPermission().getId() == permissionId) {
                        sp = p;
                    }
                }
                if (sp != null) {
                    l.remove(sp);
                }
            }
        }
        return unspecified(mapping, form, request, response);
    }
}
