/*
 *  SSL-Explorer
 *
 *  Copyright (C) 2003-2006 3SP LTD. All Rights Reserved
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2 of
 *  the License, or (at your option) any later version.
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public
 *  License along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
			
package com.sslexplorer.policyframework.wizards.forms;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.TreeSet;

import javax.servlet.http.HttpServletRequest;

import org.apache.struts.Globals;
import org.apache.struts.action.ActionErrors;
import org.apache.struts.action.ActionMapping;

import com.sslexplorer.core.BundleActionMessage;
import com.sslexplorer.policyframework.AccessRight;
import com.sslexplorer.policyframework.Permission;
import com.sslexplorer.policyframework.PolicyConstants;
import com.sslexplorer.policyframework.PolicyDatabaseFactory;
import com.sslexplorer.policyframework.ResourceType;
import com.sslexplorer.wizard.AbstractWizardSequence;
import com.sslexplorer.wizard.forms.DefaultWizardForm;

/**
 * Implementation of a {@link DefaultWizardForm}. Provides setters and getters
 * for attributes for the access rights page.
 * 
 * @author James D Robinson <a href="mailto:james@3sp.com">&lt;james@3sp.com&gt;</a>
 * 
 */
public class AccessRightsPermissionsForm extends DefaultWizardForm {

    private int selectedResourceTypeId;
    private Collection<ResourceType> resourceTypes;
    private ResourceType selectedResourceType;
    private String[] selectedAvailablePermissions;
    private List resourceTypePermissions;
    private String[] selectedResourceTypePermissions;
    private List<Permission> availablePermissions;

    /**
     * Resource type constant
     */
    public final static String ATTR_RESOURCE_TYPE = "resourceType";

    /**
     * Resource type permission constant
     */
    public final static String ATTR_RESOURCE_TYPE_PERMISSIONS = "resourceTypePermissions";

    /**
     * 
     */
    public AccessRightsPermissionsForm() {
        super(true, true, "/WEB-INF/jsp/content/policyframework/accessRightsWizard/accessRightsPermissions.jspf", "resourceType",
                        true, false, "accessRightsPermissions", "policyframework", "accessRightsWizard.permissions", 2);
    }

    /**
     * @return Returns the resourceType.
     */
    public int getSelectedResourceType() {
        return selectedResourceTypeId;
    }

    /**
     * @param resourceType The resourceType to set.
     */
    public void setSelectedResourceType(int resourceType) {
        this.selectedResourceTypeId = resourceType;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.sslexplorer.core.forms.AbstractResourceWizardForm#apply(com.sslexplorer.wizard.AbstractWizardSequence)
     */
    public void apply(AbstractWizardSequence sequence) throws Exception {
        super.apply(sequence);
        sequence.putAttribute(ATTR_RESOURCE_TYPE, new Integer(selectedResourceTypeId));
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.sslexplorer.core.forms.AbstractResourceWizardForm#init(com.sslexplorer.wizard.AbstractWizardSequence,
     *      javax.servlet.http.HttpServletRequest)
     */
    public void init(AbstractWizardSequence sequence, HttpServletRequest request) throws Exception {
        super.init(sequence, request);
        String permissionClass = (String) sequence.getAttribute(AccessRightsDetailsForm.ATTR_RESOURCE_PERMISSION_CLASS,
                        PolicyConstants.DELEGATION_CLASS);
        selectedAvailablePermissions = new String[0];
        resourceTypePermissions = (List) sequence.getAttribute(ATTR_RESOURCE_TYPE_PERMISSIONS, new ArrayList());
        availablePermissions = new ArrayList<Permission>();
        Collection<Permission> allAvailablePermissions = new TreeSet<Permission>();
        if (sequence.hasAttribute(ATTR_RESOURCE_TYPE)) {
            selectedResourceType = PolicyDatabaseFactory.getInstance().getResourceType(
                            ((Integer) sequence.getAttribute(ATTR_RESOURCE_TYPE, null)).intValue());
        }
        resourceTypes = new TreeSet<ResourceType>();
        List r = PolicyDatabaseFactory.getInstance().getResourceTypes(permissionClass);
        for (Iterator i = r.iterator(); i.hasNext();) {
            ResourceType rt = (ResourceType) i.next();
            if (rt.getPermissions().size() > 0) {
                resourceTypes.add(rt);
            }
        }
        if (selectedResourceType == null) {
            selectedResourceType = resourceTypes.iterator().next();
        }
        allAvailablePermissions.addAll(selectedResourceType.getPermissions());

        selectedResourceTypeId = selectedResourceType.getResourceTypeId();
        for (Iterator i = allAvailablePermissions.iterator(); i.hasNext();) {
            Permission p = (Permission) i.next();
            boolean found = false;
            for (Iterator j = resourceTypePermissions.iterator(); j.hasNext();) {
                AccessRight rp = (AccessRight) j.next();
                if (rp.getPermission() == p && rp.getResourceType() == selectedResourceType) {
                    found = true;
                }
            }
            if (!found) {
                availablePermissions.add(p);
            }
        }
    }

    /**
     * @return Returns the resourceTypes.
     */
    public Collection getResourceTypes() {
        return resourceTypes;
    }

    /**
     * @return Returns the resourceType.
     */
    public ResourceType getResourceType() {
        return selectedResourceType;
    }

    /**
     * @return Returns the selectedAvailablePermissions.
     */
    public String[] getSelectedAvailablePermissions() {
        return selectedAvailablePermissions;
    }

    /**
     * @param selectedAvailablePermissions The selectedAvailablePermissions to
     *        set.
     */
    public void setSelectedAvailablePermissions(String[] selectedAvailablePermissions) {
        this.selectedAvailablePermissions = selectedAvailablePermissions;
    }

    /**
     * @return Returns the resourceTypePermissions.
     */
    public List getResourceTypePermissions() {
        return resourceTypePermissions;
    }

    /**
     * @return Returns the selectedResourceTypePermissions.
     */
    public String[] getSelectedResourceTypePermissions() {
        return selectedResourceTypePermissions;
    }

    /**
     * @param selectedResourceTypePermissions The
     *        selectedResourceTypePermissions to set.
     */
    public void setSelectedResourceTypePermissions(String[] selectedResourceTypePermissions) {
        this.selectedResourceTypePermissions = selectedResourceTypePermissions;
    }

    /**
     * @return Returns the availablePermissions.
     */
    public List getAvailablePermissions() {
        return availablePermissions;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.sslexplorer.core.forms.AbstractResourceWizardForm#validate(org.apache.struts.action.ActionMapping,
     *      javax.servlet.http.HttpServletRequest)
     */
    public ActionErrors validate(ActionMapping mapping, HttpServletRequest request) {
        ActionErrors errs = super.validate(mapping, request);
        if (isCommiting() && resourceTypePermissions.size() == 0) {
            errs = errs == null ? new ActionErrors() : errs;
            errs.add(Globals.ERROR_KEY, new BundleActionMessage(getResourceBundle(),
                            "accessRightsWizard.permissions.error.noPermissionsSelected"));
        }
        return errs;
    }
}
