package com.sslexplorer.security.forms;

import java.util.Iterator;
import java.util.List;

import javax.servlet.http.HttpServletRequest;

import org.apache.struts.Globals;
import org.apache.struts.action.ActionErrors;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;

import com.sslexplorer.boot.PropertyList;
import com.sslexplorer.core.UserDatabaseManager;
import com.sslexplorer.core.forms.CoreForm;
import com.sslexplorer.security.LogonControllerFactory;
import com.sslexplorer.security.User;
import com.sslexplorer.security.UserDatabase;


/**
 * Implementation of a {@link CoreForm} that allows an administrator to 
 * create or edit a <i>Group</i> of users (previously known as a <i>Role</i>).
 * 
 * @author Brett Smith <a href="mailto: brett@3sp.com">&lt;brett@3sp.com&gt;</a>
 */
public class RoleForm extends CoreForm {
    
    // Private instance variables
    
    private String rolename;    
    private PropertyList users;
    
    /**
     * Get the list of selected users in <i>Text Field Text</i> format
     * 
     * @return selected users
     * @see PropertyList
     */
    public String getUsers() {
        return users.getAsTextFieldText();
    }
    
    /**
     * Set the list of selected users in <i>Text Field Text</i> format
     * 
     * @param users selected users
     * @see PropertyList
     */
    public void setUsers(String users) {
        this.users.setAsTextFieldText(users);
    }
    
    /**
     * Get the name of the group.
     * 
     * @return group
     */
    public String getRolename() {
        return rolename;
    }

    /**
     * Set the name of the role
     * 
     * @param rolename name of role
     */
    public void setRolename(String rolename) {
        this.rolename = rolename.trim();
    }

    /**
     * Initialise the form
     * 
     * @param users list of {@link User} objects attached to the role
     */
    public void initialize(List users) {
        rolename = "";        
        this.users = new PropertyList();
        for(Iterator i = users.iterator(); i.hasNext(); ) {
            this.users.add(((User)i.next()).getPrincipalName());
        }
    }
    
    /* (non-Javadoc)
     * @see org.apache.struts.action.ActionForm#validate(org.apache.struts.action.ActionMapping, javax.servlet.http.HttpServletRequest)
     */
    public ActionErrors validate(ActionMapping arg0, HttpServletRequest request) {
        ActionErrors errors = new ActionErrors();
        if (isCommiting()) {
           if (rolename.equals("")) {
                errors.add(Globals.ERROR_KEY, new ActionMessage("availableRoles.noRolenameSupplied"));
            }
            if (rolename.length() > 32) {
                errors.add(Globals.ERROR_KEY, new ActionMessage("availableRoles.roleNameExceeds32Chars"));
            }
            
            UserDatabase udb;
            try {
                udb = UserDatabaseManager.getInstance().getUserDatabase(
                    LogonControllerFactory.getInstance().getUser(request).getRealm().getResourceId());
                User[] u = udb.listAllUsers("*");
                List<String> users = getUserList();
                for (String user : users) {
                    if(!isAccountExists(user, u))
                        errors.add(Globals.ERROR_KEY, new ActionMessage("availableRoles.noExistingAccount", user));
                }
                if (!getEditing()) {
                    try {
                        udb.getRole(rolename);
                        errors.add(Globals.ERROR_KEY, new ActionMessage("availableRoles.roleAlreadyExists", rolename));
                    } catch (Exception e) {
                    }
                }
            } catch (Exception e1) {
                errors.add(Globals.ERROR_KEY, new ActionMessage("availableRoles.noUserDatabase"));
            }
        }
        return errors;
    }
    
    boolean isAccountExists(String accountName, User[] users) {
        for (int i = 0; i < users.length; i++) {
            if (null != users[i] && users[i].getPrincipalName().equals(accountName))
                return true;
        }
        return false;
    }

    /**
     * Get the selected as users as a list of strings
     * 
     * @return selected users as a list of strings
     */
    public List getUserList() {
        return users;
    }
}
