/*
 *  SSL-Explorer
 *
 *  Copyright (C) 2003-2006 3SP LTD. All Rights Reserved
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2 of
 *  the License, or (at your option) any later version.
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public
 *  License along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
			
package com.sslexplorer.security.forms;

import java.util.Arrays;

import javax.servlet.http.HttpSession;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.sslexplorer.core.UserDatabaseManager;
import com.sslexplorer.security.LogonControllerFactory;
import com.sslexplorer.security.Role;
import com.sslexplorer.security.RoleItem;
import com.sslexplorer.security.RoleItemModel;
import com.sslexplorer.table.forms.AbstractPagerForm;


/**
 * Implementation of {@link AbstractPagerForm} that is used to display a
 * list of configured <i>Groups</i> (previously known as <i>Roles</i>) and
 * to allow the administrator to create, edit and view (if the underlying
 * user database supports it).
 * 
 * @author Brett Smith <a href="mailto: brett@3sp.com">&lt;brett@3sp.com&gt;</a>
 */
public class ShowAvailableRolesForm extends AbstractPagerForm {

    static Log log = LogFactory.getLog(ShowAvailableRolesForm.class);

    /**
     * Constructor.
     */
    public ShowAvailableRolesForm() {
        super(new RoleItemModel());
    }

    /**
     * Initliase the form.
     * 
     * @param r roles
     * @param session session
     * @throws Exception on any error
     */
    public void initialize(Role[] r, HttpSession session) throws Exception {
        super.initialize(session, "role");
        for (int i = 0; r != null && i < r.length; i++) {
            getModel().addItem(
                            new RoleItem(r[i], Arrays.asList(UserDatabaseManager.getInstance().getUserDatabase(
                                            LogonControllerFactory.getInstance().getSessionInfo(session).getUser().getRealm()
                                                            .getResourceId()).getUsersInRole(r[i]))));
        }
        getPager().rebuild(getFilterText());
    }

}