/*
 *  SSL-Explorer
 *
 *  Copyright (C) 2003-2006 3SP LTD. All Rights Reserved
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2 of
 *  the License, or (at your option) any later version.
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public
 *  License along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
			
package com.sslexplorer.testcontainer;

import java.io.File;
import java.net.InetAddress;
import java.util.Calendar;

import javax.servlet.http.HttpSession;

import servletunit.struts.MockStrutsTestCase;

import com.sslexplorer.boot.Context;
import com.sslexplorer.core.UserDatabaseManager;
import com.sslexplorer.properties.DefaultPropertyProfile;
import com.sslexplorer.properties.Property;
import com.sslexplorer.properties.PropertyProfile;
import com.sslexplorer.properties.impl.realms.RealmKey;
import com.sslexplorer.security.Constants;
import com.sslexplorer.security.LogonController;
import com.sslexplorer.security.LogonControllerFactory;
import com.sslexplorer.security.Role;
import com.sslexplorer.security.SessionInfo;
import com.sslexplorer.security.User;

/**
 * Starting point for any Struts based unit testing 
 */
public abstract class AbstractMockStrutsTestCase extends MockStrutsTestCase
{
        private static Context context_;
        private static boolean finished_;
        private final String contextDirectory_;
        private final String configurationFile_;
        private final String userName = "test";
        private User user;

        /**
         * Default constructor
         * @param contextDirectory - path to the WEB-INF directory
         * @param configurationFile - the Struts configuration file
         */
        public AbstractMockStrutsTestCase ( String contextDirectory, String configurationFile )
        {
                contextDirectory_ = contextDirectory;
                configurationFile_ = configurationFile;
        }
        
        private static void oneTimeSetUp () throws Exception
        {
                if ( context_ == null )
                        context_ = TestContext.getTestContext ();
        }

        private static void oneTimeTearDown ()
        {
                if ( context_ != null && finished_ )
                        context_.shutdown ( false );
        }
        
        protected void setUp () throws Exception
        {
                super.setUp ();
                oneTimeSetUp ();
                setContextDirectory ( new File ( contextDirectory_ ) );
                setConfigFile ( configurationFile_ );
                getMockRequest ().setRemoteAddr ( "127.0.0.1" );
                getMockRequest ().setServletPath ( "" );
                getMockRequest ().setContextPath ( "" );
                authenticate ();
        }
        
        private void authenticate () throws Exception
        {
                LogonController logonController = LogonControllerFactory.getInstance();
                user = UserDatabaseManager.getInstance().getDefaultUserDatabase().createAccount(userName, "password", "", "Full Name", new Role[]{});
                SessionInfo nextSession = SessionInfo.nextSession ( getSession (), "logonTicket", user, InetAddress.getLocalHost (), SessionInfo.ALL_CONTEXTS, "" );
                Property.setProperty(new RealmKey("security.administrators", user.getRealm().getResourceId()), user.getPrincipalName(), nextSession);
                logonController.addSession ( "logonTicket", nextSession, request, response);
                HttpSession session = getRequest ().getSession ();
                session.setAttribute ( Constants.LOGON_TICKET, "logonTicket" );
                session.setAttribute ( Constants.SELECTED_PROFILE, getPropertyProfile () );
        }
        
        private static PropertyProfile getPropertyProfile ()
        {
                Calendar instance = Calendar.getInstance ();
                return new DefaultPropertyProfile (-1, -1, "", "", "", instance, instance );
        }

        protected void tearDown () throws Exception
        {
                super.tearDown ();
                oneTimeTearDown ();
                LogonControllerFactory.getInstance().logoff ( "logonTicket" );
                clearRequestParameters ();
                UserDatabaseManager.getInstance().getDefaultUserDatabase().deleteAccount(user);
                for (File file : TestContext.DB_DIR.listFiles()) {
                    String name = file.getName();
                    if (name.indexOf("explorer_configuration") != -1 || name.indexOf("upgrade") != -1 || name.indexOf("versions") != -1) {
                        file.delete();
                    }
                }
        }
        
        /**
         * Verifies if the supplied tile matches the actual forward.
         * @param name
         */
        public void verifyTilesForward ( String name )
        {
                assertEquals ( name, getActualForward () );
        }
        
        /**
         * A hack, should always remain the last test to do correct teardown
         */
        public void testZFinished ()
        {
                finished_ = true;
        }
}